#ifndef lint
static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
#endif
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define yyclearin (yychar=(-1))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING (yyerrflag!=0)
#define YYPREFIX "yy"
#line 2 "gram.y"
/* $Header: /local/src/CVS/nickle/gram.y,v 1.131 2004/05/20 09:37:37 bart Exp $ */

/*
 * Copyright © 1988-2004 Keith Packard and Bart Massey.
 * All Rights Reserved.  See the file COPYING in this directory
 * for licensing information.
 */

#include	"nickle.h"
#include	<stdio.h>

/*
 * This grammar generates 2 reduce/reduce and 4 shift/reduce conflicts:
 * 
 * reduce/reduce conflict on:
 *
 *	    *int . func
 *
 *	Is the '*' a dereference operator or a pointer type modifier?
 *	The grammar is ordered to make this a type modifier since
 *	the other way is less common.  Use parens when you want this.
 *
 *  reduce/reduce conflict on:
 *
 *	   &int . func
 *
 *	Is the '&' a reference operator or a reference type modifier?
 *	The grammar is ordered to make this a type modifier since
 *	the other way is less common.  Use parens when you want this.
 *
 *  shift/reduce conflict on NL:
 *
 *	    opt_nl : .
 *
 *	This says that you can have multiple NLs in a row
 *	and the grammar can't tell which opt_nl is reduced.
 *	No big deal.  (Could be fixed by replacing with opt_nls
 *      everywhere, but would change the semantics? --Bart)
 *
 *  shift/reduce conflict on CATCH:
 *
 *	That's because CATCH blocks are optional and chained.
 *	Shifting binds the CATCH to the nearest enclosing TRY,
 *	just like ELSE
 *
 *  shift/reduce conflict in ASSIGN in initializers
 *
 *	That's because struct initializers look like assignment
 *	expressions while array initializers permit any expression.
 *	Shift yields struct initialization, so the effect is
 *	to make assignment expressions invalid in array initializers
 *	(of course, you can always enclose an assignment in parens)
 *
 *  shift/reduce conflict on ELSE
 *
 *	Need we say more?
 */
int ignorenl;
int notCommand;
NamespacePtr	LexNamespace;
int funcDepth;

void ParseError (char *fmt, ...);
void yyerror (char *msg);

typedef enum _CanonTypeResult {
    CanonTypeUndefined,
    CanonTypeForward,
    CanonTypeDefined,
} CanonTypeResult;
    
static Expr *
ParseCanonFor (Expr *expr);

static CanonTypeResult
ParseCanonType (TypePtr type, Bool forwardAllowed);

static SymbolPtr
ParseNewSymbol (Publish publish, Class class, Type *type, Atom name);

#line 84 "gram.y"
typedef union {
    int		    ints;
    Value	    value;
    Class	    class;
    ArgType	    *argType;
    Type	    *type;
    Publish	    publish;
    ExprPtr	    expr;
    Atom	    atom;
    DeclListPtr	    declList;
    MemListPtr	    memList;
    Fulltype	    fulltype;
    ArgDecl	    argDecl;
    SymbolPtr	    symbol;
    NamespacePtr    namespace;
    CodePtr	    code;
    Bool	    bool;
    AtomListPtr	    atomList;
    FuncDecl	    funcDecl;
} YYSTYPE;
#line 114 "gram.c"
#define VAR 257
#define EXPR 258
#define ARRAY 259
#define STRUCT 260
#define UNION 261
#define ENUM 262
#define COMP 263
#define HASH 264
#define NL 265
#define SEMI 266
#define MOD 267
#define OC 268
#define CC 269
#define DOLLAR 270
#define DOTDOTDOT 271
#define GLOBAL 272
#define AUTO 273
#define STATIC 274
#define CONST 275
#define POLY 276
#define INTEGER 277
#define NATURAL 278
#define RATIONAL 279
#define REAL 280
#define STRING 281
#define FILET 282
#define MUTEX 283
#define SEMAPHORE 284
#define CONTINUATION 285
#define THREAD 286
#define VOID 287
#define BOOL 288
#define FUNCTION 289
#define FUNC 290
#define EXCEPTION 291
#define RAISE 292
#define TYPEDEF 293
#define IMPORT 294
#define NEW 295
#define ANONINIT 296
#define NAMESPACE 297
#define PUBLIC 298
#define PROTECTED 299
#define EXTEND 300
#define IF 301
#define ELSE 302
#define WHILE 303
#define DO 304
#define FOR 305
#define SWITCH 306
#define BREAK 307
#define CONTINUE 308
#define RETURNTOK 309
#define FORK 310
#define CASE 311
#define DEFAULT 312
#define TRY 313
#define CATCH 314
#define TWIXT 315
#define NAME 316
#define TYPENAME 317
#define NAMESPACENAME 318
#define COMMAND 319
#define NAMECOMMAND 320
#define TEN_NUM 321
#define OCTAL0_NUM 322
#define OCTAL_NUM 323
#define BINARY_NUM 324
#define HEX_NUM 325
#define TEN_FLOAT 326
#define OCTAL0_FLOAT 327
#define OCTAL_FLOAT 328
#define BINARY_FLOAT 329
#define HEX_FLOAT 330
#define CHAR_CONST 331
#define STRING_CONST 332
#define POLY_CONST 333
#define THREAD_CONST 334
#define VOIDVAL 335
#define BOOLVAL 336
#define DARROW 337
#define POUND 338
#define COMMA 339
#define ASSIGN 340
#define ASSIGNPLUS 341
#define ASSIGNMINUS 342
#define ASSIGNTIMES 343
#define ASSIGNDIVIDE 344
#define ASSIGNDIV 345
#define ASSIGNMOD 346
#define ASSIGNPOW 347
#define ASSIGNSHIFTL 348
#define ASSIGNSHIFTR 349
#define ASSIGNLXOR 350
#define ASSIGNLAND 351
#define ASSIGNLOR 352
#define QUEST 353
#define COLON 354
#define OR 355
#define AND 356
#define LOR 357
#define LXOR 358
#define LAND 359
#define EQ 360
#define NE 361
#define LT 362
#define GT 363
#define LE 364
#define GE 365
#define SHIFTL 366
#define SHIFTR 367
#define PLUS 368
#define MINUS 369
#define TIMES 370
#define DIVIDE 371
#define DIV 372
#define POW 373
#define UNIONCAST 374
#define UMINUS 375
#define BANG 376
#define FACT 377
#define LNOT 378
#define INC 379
#define DEC 380
#define STAR 381
#define AMPER 382
#define THREADID 383
#define OS 384
#define CS 385
#define DOT 386
#define ARROW 387
#define STAROS 388
#define CALL 389
#define OP 390
#define CP 391
#define POINTER 392
#define COLONCOLON 393
#define YYERRCODE 256
short yylhs[] = {                                        -1,
    0,    0,   88,   88,   91,   91,   92,   93,   90,   94,
   95,   95,   96,   96,   97,   97,   89,   89,   89,   89,
   89,   89,    2,    2,    4,    4,    3,    6,    6,    6,
    5,    5,   21,   21,    1,    1,   22,   22,   53,   53,
   98,   99,    7,  100,  101,   10,   10,   11,   11,   11,
   11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
   11,   11,   11,   11,   11,  102,   11,   11,   11,   11,
   63,   63,   12,   12,   13,    8,    8,  103,    9,   14,
   14,   15,   16,   16,   17,   17,   18,   19,   19,   20,
   20,   31,   31,   27,   27,   29,   29,   34,   34,   28,
   28,   30,   37,   32,   33,   33,   33,   35,   35,   35,
   36,   36,   36,   36,   36,   36,   36,   40,   40,   40,
   40,   40,   40,   39,   39,   39,   41,   41,   41,   41,
   41,   41,   38,   38,   45,   45,   45,   45,   45,   45,
   45,   45,   45,   45,   45,   45,   42,   42,   43,   43,
   44,   44,   46,   46,   47,   47,   48,   48,   49,   49,
   49,   49,   50,   50,   51,   51,   52,   52,   54,   54,
   55,   55,   56,   56,   59,   57,   57,   58,   58,   60,
   60,   61,   61,   62,   62,   64,   64,   71,   71,   65,
   65,   66,   66,   69,   69,   70,   70,   67,   67,   67,
   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,
   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,
   67,   67,   67,   67,   67,   67,   67,   67,   67,   67,
   67,   67,   72,   72,   72,   72,   72,   72,   72,   72,
   72,   72,   72,   72,   72,   68,   68,   68,   68,   68,
   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,
   68,   68,   68,   68,   68,   68,   68,   68,   68,   68,
   68,   68,   68,   68,   68,   68,   68,   73,   73,   74,
   74,   74,   74,   74,   75,   75,   75,   76,   76,   23,
   23,   24,   24,   25,   26,   77,   77,   78,   78,   79,
   79,   79,   80,   81,   81,   82,   82,   83,   83,   84,
   85,   85,   86,   86,   87,   87,   87,   87,
};
short yylen[] = {                                         2,
    2,    0,    1,    3,    1,    1,    0,    0,    0,    0,
    1,    0,    1,    0,    1,    0,    4,    6,    4,    6,
    6,    1,    1,    0,    3,    1,    2,    1,    1,    1,
    3,    0,    2,    1,    2,    1,    3,    2,    1,    1,
    0,    0,    3,    2,    2,    2,    0,    9,   11,    9,
   10,    9,    9,   10,    2,    2,    3,    2,    1,    1,
    3,    9,    6,    6,    7,    0,    9,    6,    5,   11,
    3,    1,    2,    0,    6,    1,    1,    0,    2,    1,
    3,    3,    2,    0,    4,    3,    3,    2,    0,    7,
    3,    1,    0,    3,    1,    3,    1,    1,    1,    4,
    2,    1,    1,    4,    3,    4,    4,    2,    2,    0,
    5,    4,    4,    3,    4,    3,    3,    2,    4,    4,
    4,    4,    0,    2,    2,    2,    1,    4,    4,    4,
    3,    1,    1,    0,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    0,    3,    1,
    3,    1,    3,    1,    4,    0,    4,    0,    1,    1,
    1,    1,    1,    0,    1,    1,    1,    1,    3,    2,
    3,    2,    2,    1,    3,    1,    0,    3,    2,    2,
    1,    1,    0,    1,    0,    1,    2,    1,    3,    1,
    0,    3,    1,    1,    0,    3,    2,    1,    3,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    5,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    2,    6,    8,    8,    8,    3,    6,    2,    1,    3,
    3,    4,    4,    4,    3,    3,    6,    1,    0,    1,
    1,    1,    1,    1,    1,    2,    0,    5,    9,    2,
    1,    3,    1,    1,    0,    3,    1,    1,    1,    1,
    2,    0,    4,    3,    1,    3,    2,    1,    1,    4,
    3,    1,    3,    3,    1,    1,    1,    2,
};
short yydefred[] = {                                      2,
    0,    9,   22,    1,    3,    0,    0,    0,    0,    0,
   60,    0,   41,    0,  159,  160,  161,  162,  135,  136,
  137,  138,  139,  140,  141,  142,  143,  144,  145,  146,
    7,    0,  165,  166,  168,    7,    7,    7,    7,    7,
    0,    0,    0,    0,    7,    7,   39,   34,    0,    0,
    0,  247,  249,  248,  250,  251,  252,  254,  253,  255,
  256,  257,  258,  259,  260,    0,    0,    0,    0,    0,
    0,    0,    0,  269,    0,  246,   59,    9,  132,    0,
    0,   41,    0,    0,    0,    0,  127,    0,    0,    0,
    0,   36,    0,    0,    0,    0,    0,    5,    6,    4,
    0,    0,    7,    0,  280,  282,  281,  283,  284,  200,
   44,  268,  278,    0,    0,    0,   41,   41,   41,   41,
   41,   55,   56,    0,    0,    0,    0,  184,    0,    0,
    0,   41,   38,    9,  190,    0,    9,    0,   23,    0,
  126,    0,    0,  125,    0,    0,    0,    0,    0,  261,
    0,    0,    0,    0,    0,   33,    0,   35,   77,   42,
   76,    0,    0,    0,    0,  187,    0,   41,    0,    0,
    0,    0,  124,    0,    0,    0,    0,    0,    0,    7,
    0,    7,    7,    7,    7,    0,    0,    7,   58,    0,
    0,    0,  245,  233,  234,  235,  236,  237,  238,  239,
  240,  241,  242,  243,  244,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  206,  208,  210,    0,    0,
    0,    0,    0,    0,    0,   42,    0,    0,    0,  133,
    0,    0,    0,   41,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  102,   57,   74,    0,    0,    0,    0,
   32,   28,   30,   29,    0,    0,    0,    0,    0,    0,
  271,    0,    0,  270,   11,   19,   37,   61,    0,   80,
   79,    0,    0,    0,  176,    0,    0,  105,    0,    0,
    0,  266,  114,  152,    0,    0,    0,    0,    0,  170,
    0,    0,    0,  118,  126,  125,    0,    0,    0,  116,
    0,    0,    0,    0,    7,    0,  117,    0,    9,   17,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  275,  276,    0,    0,    0,  194,    0,
    0,   43,   46,    0,  128,    0,  129,    0,    0,  130,
  107,    0,    0,    0,    0,    0,    0,    0,    0,   13,
    0,  192,    0,   25,   31,    0,    0,    0,    0,    0,
    0,    0,    0,  180,  104,  182,    0,  179,  106,    0,
    0,  315,  317,  316,  109,  103,    0,    0,   78,    0,
    0,    0,    0,    0,  173,  169,    0,  172,  131,  112,
   41,   99,    0,    0,    0,    0,    0,  113,    0,  115,
   40,   66,    0,    0,  273,  272,    0,  197,  274,   45,
    0,    0,    0,   94,    0,    0,    0,    0,    0,    0,
    0,    0,   73,   69,    0,   20,   21,   41,  149,   41,
   41,  153,    0,   42,   81,  178,  318,    0,    0,    0,
    0,    0,  297,    0,  305,    0,  312,  299,  100,    0,
   42,  122,  119,  151,  120,  121,  171,    0,    8,    0,
   98,    0,    8,  111,    0,   18,    0,  196,  155,  157,
    0,   63,    0,    0,    0,    0,    0,    0,   41,    0,
    0,    0,    0,    0,  262,    0,    0,  307,  309,    0,
    0,    0,    0,    0,    0,    0,    0,  175,  277,   42,
   64,   96,    8,   68,    0,    0,    0,   42,    0,   71,
   42,   42,    0,    0,    0,    0,   42,  300,    0,   42,
    0,   42,  285,    0,  314,  294,  295,    0,  306,    0,
  296,    0,    0,  304,  303,    0,  311,  310,    0,   65,
    0,   42,    0,    0,    8,    8,   42,    8,    8,    0,
    0,   42,    0,    0,    0,  301,  265,  263,  286,  264,
    0,    0,  288,    8,    8,    8,   41,    0,   42,    0,
   42,   48,   50,    8,   52,   53,    0,    0,   82,   83,
   42,   42,    0,  292,   62,   67,   54,    0,    0,   87,
   88,    8,   51,    0,   86,   75,    8,    0,  291,   42,
    0,   91,   49,   85,   70,  290,    0,   92,    0,  289,
    0,   42,   90,
};
short yydgoto[] = {                                       1,
   76,  137,  138,  139,  140,  265,   77,  160,  161,  236,
  237,  368,  443,  281,  532,  572,  573,  562,  589,  590,
   79,   80,  620,  547,  548,  581,  246,  166,  414,  167,
  629,   81,   82,  415,  290,   83,  397,   84,  129,  173,
   86,  296,  268,  298,   87,  269,  241,  243,   88,   89,
   90,   91,   92,  174,  302,  303,  284,  285,  399,  286,
  388,  439,  440,  238,  134,  135,   94,   95,  348,  349,
   96,  229,  112,  110,  542,  392,  462,  463,  537,  393,
  464,  465,  508,  394,  466,  467,  468,    4,    5,    7,
  100,  170,  444,    6,  276,  371,  513,  111,  351,   97,
  352,  485,  162,
};
short yysindex[] = {                                      0,
   38,    0,    0,    0,    0, 1106,   10, -209,  -11, -187,
    0,  199,    0,  199,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   21,    0,    0,    0,    0,    0,    0,    0,    0,
 -167, -151, 1469, 2576,    0,    0,    0,    0, -255, 2576,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2576, 2576, 2576, 2576, 2576,
 2576, 2576, -237,    0, 1348,    0,    0,    0,    0,   -8,
 -108,    0, -116, -122, -185, -113,    0, 2828,   27, 2769,
  -25,    0, -227, 4945,  193,    1, 1227,    0,    0,    0,
 2828, 2828,    0,    9,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   40,  -13,   37,    0,    0,    0,    0,
    0,    0,    0,   91,   46, 2769,  114,    0, -185,  461,
 1227,    0,    0,    0,    0, 3179,    0,   44,    0,  150,
    0, -189, -189,    0, -189, -189, -189, -189, -189,    0,
 1971,   12,  -79,   22,  153,    0,   36,    0,    0,    0,
    0, -221, 2828,  117,   57,    0,  122,    0,  155,  153,
 2092,  859,    0, -113, -255, 2828, 2828, 2828,  -19,    0,
  153,    0,    0,    0,    0, 2828,  153,    0,    0, 1469,
  213, 2576,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2576, 2576, 2576, 2576, 2576,
 2576, 2576, 2576, 2576, 2576, 2576, 2576, 2576, 2576, 2576,
 2576, 2576, 2576, 2576, 2576,    0,    0,    0, 2576, 2576,
  163,  164, 2576, 2576, 2576,    0, 1227,  215,    9,    0,
  217,    9,  226,    0,  162,  234,  124, 2576,  127,  137,
 1227,  138,  142,    0,    0,    0,  143,  268, 2576,  268,
    0,    0,    0,    0,  144, 2576,  -69, -328,  151, 4783,
    0,  222,    0,    0,    0,    0,    0,    0, 2576,    0,
    0,  225,    0,  156,    0, -222,  152,    0, 2213,  205,
  159,    0,    0,    0,  -37,  166,  207, -305,  167,    0,
  237,  170, -218,    0,    0,    0,  171,   36,  153,    0,
  238, 2815,   21,  153,    0,  153,    0,   71,    0,    0,
  133, 4838,  585,  966, 1090,  311,  347,  795,  795,  283,
  283,  283,  283,  709,  709,   78,   78,  133,  133,  133,
  133, 4945,  172,    0,    0,  181, 4731,  176,    0, 4945,
  302,    0,    0,  310,    0,  316,    0,  194,    9,    0,
    0,  195, 1469, 1469,  282, 1469, 1469,  273, 1469,    0,
  323,    0,  324,    0,    0,  200,  223,  203,  206, 2576,
  208,  328, 2997,    0,    0,    0, 2828,    0,    0, 1596,
 4945,    0,    0,    0,    0,    0,   46, 2828,    0, -113,
 -113,  327, -113, -113,    0,    0, 2828,    0,    0,    0,
    0,    0,    0,  334,  262,   56,  336,    0,  153,    0,
    0,    0,  340, 2576,    0,    0, 2576,    0,    0,    0,
 2828, 2828, 1469,    0,  341,  218,  221,  216,  349,  232,
  233,   21,    0,    0,  350,    0,    0,    0,    0,    0,
    0,    0, 2688,    0,    0,    0,    0,  286, 2213, -237,
 4890,  274,    0,  288,    0,  289,    0,    0,    0,  240,
    0,    0,    0,    0,    0,    0,    0,  242,    0,   56,
    0,  367,    0,    0,  366,    0,  461,    0,    0,    0,
  244,    0, 1227, 1227, 1469, 1469, 1227,  371,    0, 1469,
  372,  373,  374,  193,    0, 2213, 4945,    0,    0,  329,
 2213, 2213,  376, 2334,  375,  342,  388,    0,    0,    0,
    0,    0,    0,    0, 1227,  371,  359,    0,  252,    0,
    0,    0,   84,  159,  275, 1723,    0,    0, 2455,    0,
 1850,    0,    0, 4945,    0,    0,    0,  326,    0, 4945,
    0,  398, 4890,    0,    0,  286,    0,    0,  402,    0,
  417,    0,  110, 1227,    0,    0,    0,    0,    0, 1469,
  335,    0,   84,  371, 1227,    0,    0,    0,    0,    0,
  303,  329,    0,    0,    0,    0,    0,  338,    0,  110,
    0,    0,    0,    0,    0,    0,  339, 1227,    0,    0,
    0,    0, -176,    0,    0,    0,    0,  378, 1227,    0,
    0,    0,    0, 1227,    0,    0,    0, 2213,    0,    0,
  379,    0,    0,    0,    0,    0,  428,    0,  344,    0,
 1227,    0,    0,
};
short yyrindex[] = {                                      0,
  985,    0,    0,    0,    0,  115,    0,    0,    0,    0,
    0,    0,    0, 2882,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   -4,  409,    0,    0,    0,    0, 2937,  -87,
 -123,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  409,  409,  409,  409,  409,
  409,  409,    0,    0,  409,    0,    0,    0,    0,    0,
 -166,    0,    0,    0,   15,   25,    0,  -16,  403,  126,
    0,    0,  436, -215, 3246,  -80,   85,    0,    0,    0,
  -62,  -27,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  -63,    0,    0,  412, 1678,
  115,    0,    0,    0,    0, -235,    0,  187,    0,    0,
    0, 3363, 3418,    0, 3473, 3528, 3583, 3638, 3693,    0,
  409,    0, -229,    0,  725,    0, 2937,    0,    0,    0,
    0,    0, -301,    0, -134,    0, -210,    0,    0,  -24,
 -271,    0,    0,   25,    0,    0,    0,    0,    0,    0,
  -24,    0,    0,    0,    0,  -16,  -24,    0,    0,  409,
    0,  409,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  409,  409,  409,  409,  409,
  409,  409,  409,  409,  409,  409,  409,  409,  409,  409,
  409,  409,  409,  409,  409,    0,    0,    0,  409,  409,
    0,    0,  409, -265,  409,    0,  214,    0,    0,    0,
    0,    0,    0,    0,   -5,    0,    0, -264,    0,    0,
  115,    0,    0,    0,    0,    0,    0,  438,  409,  438,
    0,    0,    0,    0, -103, -238,  412,    0,    0,  319,
    0,    0,   54,    0,    0,    0,    0,    0,  409,    0,
    0,    0, -231,    0,    0,  315,    0,    0,  409,  -57,
    0,    0,    0,    0,  412,    0,  337,    0,    0,    0,
 -250,    0,  315,    0,    0,    0,    0,    0,  -24,    0,
    0,    0,    0,  -24,    0,  -24,    0,    0,    0,    0,
 3748,    0, 1923, 1549, 1802, 4573, 4518, 4408, 4463,  841,
 4243, 4298, 4353, 4133, 4188, 4023, 4078, 3803, 3858, 3913,
 3968,   13,    0,    0,    0,    0,  315,    0,    0, -202,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  409,  409,    0, -257,  409,  477,   -4,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  409,
 4676,    0,    0,    0,    0,    0,  405,    0,    0,  409,
 -156,    0,    0,    0,    0,    0,    0, -301,    0,   25,
   25,    0,   25,   25,    0,    0,    0,    0,    0,    0,
    0,    0, -223,    0,  458,    0,    0,    0,  -24,    0,
    0,    0,    0,  409,    0,    0,  409,    0,    0,    0,
  -62,  -27,  409,    0,    0,    0,    0,    0,  343,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  409,    0,    0,    0,    0, 4624,  409,  415,
 -204,  161,    0,  463,    0,  463,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2044,    0,    0,    0,
    0,    0,  115,  115,  409, -257,  115,    0,    0, -258,
 3064,    0, 3119, 3301,    0,  409, -241,    0,    0,    0,
  409,   -2,  466,   -3,    0,  467,    0,    0,    0,    0,
    0,    0,    0,    0,   85,    0,  604,    0,    0,    0,
    0,    0,  472,    0,    0,  409,    0,    0,  409,    0,
  409,    0,    0, -190,    0,    0,    0,  363,    0, -204,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  486,  115,    0,    0,    0,    0,    0,  409,
    0,    0,  472,    0,  115,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  486,
    0,    0,    0,    0,    0,    0,    0,  214,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  214,    0,
    0,    0,    0,  214,    0,    0,    0,  409,    0,    0,
  418,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  214,    0,    0,
};
short yygindex[] = {                                      0,
  -28,    0,    0,  512,    0,    0,  -74,    0,  392, -224,
   -1,    0,    0,    0,    0,  265,    0,    0,  220,    0,
    0,  298,    0,  229,    0,    0, -219,  442, -303,    0,
    0,    0,    0,    0,    0,  -40,    0,  -85,   -6, -160,
    0,    0,  644,    0,    0, -133,  404,  410,  165,    0,
  -31,    0,  -70,  365,  437,    0,  447,  462,  314,    0,
 -281,  -41,  354,    2,  603,  595,    4,  406,    0,  431,
    0,    0,    0,  848,    0,  -30,    0, -506,    0,  381,
    0,  361,  357,    0,    0,  383, -260,    0,    0,  -51,
    0, 1028,   52,    0, -163,  640,  -23,  -14,   28, -329,
 -531,    0,    0,
};
#define YYTABLESIZE 5325
short yytable[] = {                                      85,
  152,  127,  125,  115,   78,  551,  293,   93,  185,  158,
  377,  126,  353,  304,  134,  239,  242,  310,  134,  354,
  174,  408,  356,  317,  134,  134,  155,  308,  395,  193,
  193,  134,  134,  402,  125,    7,   85,  299,  189,  181,
  599,  191,   98,  126,  128,  158,   13,  130,  386,  188,
  188,  134,  386,  136,  110,  110,  378,  610,  101,  141,
  133,  144,  189,  189,  298,  428,  298,  163,  153,  142,
  143,  145,  146,  147,  148,  149,  154,  282,  313,  403,
  104,  180,  258,  185,  133,  260,    7,  280,  174,  177,
   85,   13,   34,   34,  240,  240,  343,  308,  122,  346,
  150,   78,  249,  250,  251,  252,  253,  181,  108,  108,
  190,  626,  482,  148,  123,   98,  387,  257,  279,  185,
  407,  454,  188,  188,   85,  195,  191,  110,  110,  256,
  102,  102,  185,  185,  298,  189,  189,  133,  188,  434,
  174,   24,   24,  110,  267,  410,  150,  150,  313,  125,
  418,  189,  420,  291,  270,  193,  283,  159,  126,  181,
   34,   27,   27,  618,  295,  301,   34,  168,  533,  305,
  306,  307,  164,   78,  270,  188,  522,  191,  191,  315,
  110,  108,  108,   85,  186,  186,  226,  278,  189,  227,
  228,  319,   32,   32,   32,  321,  563,  108,  509,  165,
  169,    7,  134,  102,  102,  102,  156,  101,  101,  322,
  323,  324,  325,  326,  327,  328,  329,  330,  331,  332,
  333,  334,  335,  336,  337,  338,  339,  340,  341,  358,
   85,  552,  342,  270,  108,   27,  270,  347,  350,  472,
  473,  158,  475,  476,   85,  545,  452,  422,    7,  365,
  509,  136,    7,  134,  186,  484,  102,  186,  382,  144,
   95,  185,  136,   95,   12,   15,   15,  423,   15,  145,
  171,  188,    7,  186,   98,   99,  172,  199,  199,    7,
  101,  199,  383,  199,  417,  134,  134,  134,  134,  123,
  186,   12,  391,    2,  103,  123,  101,  156,  308,    7,
  561,  282,    3,    7,  133,  416,  272,   47,  156,  157,
  186,  273,  282,  123,  123,  376,  169,  182,  131,  183,
  184,   41,  125,  125,  245,  125,  125,  445,  125,  116,
    7,  126,  126,  101,  126,  126,   47,  126,   49,  235,
  123,  123,  131,  131,  192,  239,  242,  400,  169,  199,
  199,  199,   47,   47,  157,  247,   85,   85,  102,   85,
   85,  254,   85,  123,  436,  437,  199,  128,  441,  131,
  128,  412,  481,  615,  134,  164,  248,  164,  164,  255,
  283,  164,  261,  270,  622,  179,   47,  179,  421,  624,
    7,  283,  125,  461,  570,  571,  478,  199,  179,  179,
  301,  126,  271,  199,  134,  164,  632,  164,  164,  123,
  123,  164,  274,  499,    7,  123,  163,  275,  163,  163,
  587,  588,  163,  179,  240,  240,   85,  487,  277,   16,
  347,   16,  287,  501,  491,  502,  503,  131,  131,  131,
  515,    7,  517,  131,  131,  510,  288,  222,  223,  224,
  225,   26,   26,  226,  125,  125,  227,  228,  535,  125,
  179,  289,  507,  126,  126,  262,  263,  264,  126,  179,
  292,  540,  543,  179,  179,  179,    8,  320,  344,  345,
  189,  505,   47,  179,  534,  355,   85,   85,   85,   85,
   85,  527,  528,   85,  357,  531,  529,  128,  519,  601,
  359,  128,  360,  134,  164,  225,  164,  164,  226,  544,
  164,  227,  228,  361,  507,  550,  363,  553,   85,  105,
  106,  107,  108,  109,   47,   47,  364,  366,  619,  125,
  521,  367,  369,  370,  524,  379,  375,  381,  126,  553,
  384,  389,  550,  396,  550,  377,  385,  559,  398,  192,
  401,  404,  405,  411,  565,  566,  425,   85,  568,  569,
  406,  409,  591,   85,  577,  426,  429,  578,   85,  580,
  430,  597,  608,  602,  560,  431,  230,  192,  231,  232,
  233,  432,  234,  433,  438,  435,  442,  446,  447,  586,
  448,   85,  449,  450,  594,  390,  451,  474,  453,  479,
  480,  483,   85,   42,  486,  495,  492,   85,  493,  179,
  116,  494,  512,  192,  496,  500,  592,  593,  612,  595,
  596,  550,  497,  498,   85,  506,  514,  516,  616,  617,
  518,  172,  523,  525,  526,  605,  606,  607,   13,  536,
  539,  541,  567,  555,  546,  613,  386,  627,  218,  219,
  220,  221,  222,  223,  224,  225,  558,  556,  226,  633,
  564,  227,  228,  623,  582,  575,  583,  584,  625,  211,
  212,  213,  214,  215,  216,  217,  218,  219,  220,  221,
  222,  223,  224,  225,  179,  585,  226,  603,  598,  227,
  228,  609,  614,  621,  628,  179,  630,  631,  134,  167,
    9,  133,   14,  154,  179,  183,  212,  213,  214,  215,
  216,  217,  218,  219,  220,  221,  222,  223,  224,  225,
  134,  147,  226,   97,   12,  227,  228,  192,  179,  179,
   41,   16,    8,   72,  183,   15,    8,    8,    8,  116,
   84,    8,    8,    8,    8,    8,    8,  293,    8,    8,
    8,    8,    8,    8,   89,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,   93,  374,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,    8,
  471,    8,    8,    8,    8,    8,    8,    8,    8,    8,
    8,    8,    8,    8,    8,    8,    8,    8,    8,  611,
  604,    8,    8,  206,  297,  207,  208,  209,  210,  211,
  212,  213,  214,  215,  216,  217,  218,  219,  220,  221,
  222,  223,  224,  225,  489,    8,  226,  600,  469,  227,
  228,  490,  520,  477,  470,    8,    8,  574,  456,  530,
  362,  192,    8,  372,    8,    8,    8,  488,  504,   42,
    8,  113,    8,   42,   42,   42,    8,  549,   42,   42,
   42,   42,   42,   42,  554,   42,   42,   42,   42,   42,
   42,  538,   42,   42,   42,   42,   42,   42,   42,   42,
   42,   42,   42,   42,   42,   42,   42,   42,  557,  373,
   42,   42,   42,   42,   42,    0,   42,   42,   42,   42,
   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,
   42,   42,   42,   42,   42,   42,   42,   42,   42,   42,
   42,   42,   42,   42,   42,   42,    0,    0,   42,   42,
  208,  209,  210,  211,  212,  213,  214,  215,  216,  217,
  218,  219,  220,  221,  222,  223,  224,  225,    0,    0,
  226,    0,   42,  227,  228,    0,    0,    0,    0,    0,
    0,    0,   42,   42,    0,  192,    0,    0,    0,   42,
   12,   42,   42,   42,   12,   12,   12,   42,    0,   42,
   12,   12,   12,   42,   12,    0,   12,   12,   12,   12,
   12,   12,    0,   12,   12,   12,   12,   12,   12,   12,
   12,   12,   12,   12,   12,   12,   12,   12,   12,    0,
    0,   12,   12,   12,   12,   12,    0,   12,   12,   12,
   12,   12,   12,   12,   12,    0,    0,   12,    0,   12,
   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
   12,   12,   12,   12,   12,   12,   12,    0,  114,   12,
   12,  192,    0,  117,  118,  119,  120,  121,    0,    0,
    0,    0,  131,  132,    0,    0,  220,  221,  222,  223,
  224,  225,    0,   12,  226,    0,    0,  227,  228,    0,
    0,    0,    0,   12,   12,    0,    0,    0,    0,    0,
   12,    0,   12,   12,   12,  229,  229,    0,   12,  229,
   12,  229,    0,    0,   12,  181,    0,  187,    8,  124,
   10,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  244,    0,    0,    0,   19,   20,    0,   21,   22,   23,
   24,   25,   26,   27,   28,   29,   30,    0,    0,    0,
    0,    0,    0,  187,    0,    0,  214,  215,  216,  217,
  218,  219,  220,  221,  222,  223,  224,  225,    0,    0,
  226,    0,    0,  227,  228,   48,  175,  229,  229,  229,
  229,  229,  229,  229,  229,  229,  229,  229,  229,  229,
  229,  229,  229,  229,  229,  229,  229,  229,  229,  229,
  229,  229,  229,  229,  229,  229,    0,  309,    0,  311,
  312,  313,  314,  316,    0,  318,    0,  176,    0,    0,
    0,    0,    0,    0,    0,  229,    0,    0,  177,    0,
    0,  229,  192,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   10,   10,   10,    0,  178,  300,
   10,   10,   10,    0,   10,    0,   10,   10,   10,   10,
   10,   10,    0,   10,   10,   10,   10,   10,   10,   10,
   10,   10,   10,   10,   10,   10,   10,   10,   10,    0,
    0,   10,   10,   10,   10,   10,    0,   10,   10,   10,
   10,   10,   10,   10,   10,    0,    0,   10,    0,   10,
   10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
   10,   10,   10,   10,   10,   10,   10,    0,    0,   10,
   10,    0,  209,  210,  211,  212,  213,  214,  215,  216,
  217,  218,  219,  220,  221,  222,  223,  224,  225,    0,
    0,  226,  419,   10,  227,  228,    0,    0,    0,    0,
    0,    0,    0,   10,   10,    0,  192,    0,    0,    0,
   10,    0,   10,   10,   10,    8,    9,   10,   10,    0,
   10,   11,   12,   13,   10,   14,    0,   15,   16,   17,
   18,   19,   20,    0,   21,   22,   23,   24,   25,   26,
   27,   28,   29,   30,   31,    0,    0,   32,    0,    0,
    0,    0,    0,   33,   34,   35,   36,    0,   37,   38,
   39,   40,   41,   42,   43,   44,    0,    0,   45,    0,
   46,   47,   48,   49,   50,   51,   52,   53,   54,   55,
   56,   57,   58,   59,   60,   61,   62,   63,    0,    0,
   64,   65,    0,    0,    0,    0,    0,  210,  211,  212,
  213,  214,  215,  216,  217,  218,  219,  220,  221,  222,
  223,  224,  225,    0,   66,  226,    0,    0,  227,  228,
    0,    0,    0,    0,   67,   68,    0,    0,    0,    0,
    0,   69,    0,   70,   71,   72,    8,    9,   10,   73,
    0,   74,   11,   12,   13,   75,   14,    0,   15,   16,
   17,   18,   19,   20,    0,   21,   22,   23,   24,   25,
   26,   27,   28,   29,   30,   31,    0,    0,   32,    0,
    0,    0,    0,    0,   33,   34,   35,   36,    0,   37,
   38,   39,   40,   41,   42,   43,   44,    0,    0,   45,
    0,   46,   47,   48,   49,    0,    0,   52,   53,   54,
   55,   56,   57,   58,   59,   60,   61,   62,   63,    0,
    0,   64,   65,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   66,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   67,   68,    0,    0,    0,
    0,    0,   69,    0,   70,   71,   72,    8,  124,   10,
   73,    0,   74,    0,   12,   13,   75,   14,    0,   15,
   16,   17,   18,   19,   20,    0,   21,   22,   23,   24,
   25,   26,   27,   28,   29,   30,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   33,   34,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   44,    0,    0,
    0,    0,    0,   47,   48,   49,    0,    0,   52,   53,
   54,   55,   56,   57,   58,   59,   60,   61,   62,   63,
    0,    0,   64,   65,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   66,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   67,   68,    0,    0,
    0,    0,    0,   69,    0,   70,   71,   72,    8,  124,
   10,  151,    0,   74,    0,   12,    0,   75,   14,    0,
   15,   16,   17,   18,   19,   20,    0,   21,   22,   23,
   24,   25,   26,   27,   28,   29,   30,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   33,   34,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   44,    0,
    0,    0,    0,    0,   47,   48,   49,    0,    0,   52,
   53,   54,   55,   56,   57,   58,   59,   60,   61,   62,
   63,    0,    0,   64,   65,    0,    0,    0,    0,    0,
    0,    0,    0,  225,  225,    0,    0,  225,    0,  225,
    0,    0,    0,    0,    0,    0,    0,   66,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   67,   68,    0,
    0,    0,    0,    0,   69,    0,   70,   71,   72,    0,
    0,    0,   73,    0,   74,    8,  124,   10,   75,    0,
    0,    0,   12,  390,  457,   14,    0,    0,    0,    0,
    0,   19,   20,    0,   21,   22,   23,   24,   25,   26,
   27,   28,   29,   30,    0,  225,  225,  225,  225,  225,
  225,  225,  225,  225,  225,  225,  225,  225,  225,  225,
  225,  225,  225,  225,  225,   44,    0,    0,    0,    0,
    0,  458,   48,   49,    0,    0,   52,   53,   54,   55,
   56,   57,   58,   59,   60,   61,   62,   63,    0,    0,
   64,   65,  459,  225,    0,    0,    0,    0,    0,  225,
    0,    0,  211,  211,    0,    0,  211,    0,  211,    0,
    0,    0,    0,    0,   66,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   67,   68,    0,    0,    0,    0,
    0,   69,    0,   70,   71,   72,    0,    0,    0,  460,
    0,   74,    8,  124,   10,   75,    0,    0,    0,   12,
    0,  576,   14,    0,    0,    0,    0,    0,   19,   20,
    0,   21,   22,   23,   24,   25,   26,   27,   28,   29,
   30,    0,    0,    0,  211,  211,  211,  211,  211,  211,
  211,  211,  211,  211,  211,  211,  211,  211,  211,  211,
    0,  211,   44,    0,    0,    0,    0,    0,   47,   48,
   49,    0,    0,   52,   53,   54,   55,   56,   57,   58,
   59,   60,   61,   62,   63,    0,    0,   64,   65,  459,
    0,    0,  211,    0,    0,    0,  224,  224,  211,    0,
  224,    0,  224,    0,    0,    0,    0,    0,    0,    0,
    0,   66,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   67,   68,    0,    0,    0,    0,    0,   69,    0,
   70,   71,   72,    0,    0,    0,   73,    0,   74,    8,
  124,   10,   75,    0,    0,    0,   12,  390,  579,   14,
    0,    0,    0,    0,    0,   19,   20,    0,   21,   22,
   23,   24,   25,   26,   27,   28,   29,   30,  224,  224,
  224,  224,  224,  224,  224,  224,  224,  224,  224,  224,
  224,  224,  224,  224,  224,  224,  224,  224,  224,   44,
    0,    0,    0,    0,    0,   47,   48,   49,    0,    0,
   52,   53,   54,   55,   56,   57,   58,   59,   60,   61,
   62,   63,    0,    0,   64,   65,  224,  226,  226,    0,
    0,  226,  224,  226,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   66,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   67,   68,
    0,    0,    0,    0,    0,   69,    0,   70,   71,   72,
    8,  124,   10,  460,    0,   74,    0,   12,    0,   75,
   14,    0,    0,    0,    0,    0,   19,   20,    0,   21,
   22,   23,   24,   25,   26,   27,   28,   29,   30,  226,
  226,  226,  226,  226,  226,  226,  226,  226,  226,  226,
  226,  226,  226,  226,  226,  226,  226,  226,    0,    0,
   44,    0,    0,    0,    0,    0,   47,   48,   49,    0,
    0,   52,   53,   54,   55,   56,   57,   58,   59,   60,
   61,   62,   63,    0,    0,   64,   65,  226,  221,  221,
    0,    0,  221,  226,  221,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   66,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   67,
  266,    0,    0,    0,    0,    0,   69,    0,   70,   71,
   72,    8,  124,   10,   73,  150,   74,    0,   12,    0,
   75,   14,  294,    0,    0,    0,    0,   19,   20,    0,
   21,   22,   23,   24,   25,   26,   27,   28,   29,   30,
  221,  221,  221,  221,  221,  221,  221,  221,  221,  221,
  221,  221,  221,  221,  221,  221,    0,  221,    0,    0,
    0,   44,    0,    0,    0,    0,    0,   47,   48,   49,
    0,    0,   52,   53,   54,   55,   56,   57,   58,   59,
   60,   61,   62,   63,    0,    0,   64,   65,  221,    0,
    0,    0,    0,    0,  221,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   66,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   67,  266,    0,    0,    0,    0,    0,   69,    0,   70,
   71,   72,    8,  124,   10,   73,    0,   74,    0,   12,
  390,   75,   14,    0,    0,    0,    0,    0,   19,   20,
    0,   21,   22,   23,   24,   25,   26,   27,   28,   29,
   30,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   44,    0,    0,    0,    0,    0,   47,   48,
   49,    0,    0,   52,   53,   54,   55,   56,   57,   58,
   59,   60,   61,   62,   63,    0,    0,   64,   65,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   66,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   67,   68,    0,    0,    0,    0,    0,   69,    0,
   70,   71,   72,    8,  124,   10,   73,    0,   74,    0,
   12,    0,   75,   14,    0,    0,    0,    0,    0,   19,
   20,    0,   21,   22,   23,   24,   25,   26,   27,   28,
   29,   30,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   44,    0,    0,    0,    0,    0,   47,
   48,   49,    0,    0,   52,   53,   54,   55,   56,   57,
   58,   59,   60,   61,   62,   63,    0,    0,   64,   65,
  459,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   66,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   67,   68,    0,    0,    0,    0,    0,   69,
    0,   70,   71,   72,    8,  124,   10,   73,    0,   74,
    0,   12,  390,   75,   14,    0,    0,    0,    0,    0,
   19,   20,    0,   21,   22,   23,   24,   25,   26,   27,
   28,   29,   30,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   44,    0,    0,    0,    0,    0,
   47,   48,   49,    0,    0,   52,   53,   54,   55,   56,
   57,   58,   59,   60,   61,   62,   63,    0,    0,   64,
   65,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   66,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   67,   68,    0,    0,    0,    0,    0,
   69,    0,   70,   71,   72,    8,  124,   10,  460,    0,
   74,    0,   12,    0,   75,   14,    0,    0,    0,    0,
    0,   19,   20,    0,   21,   22,   23,   24,   25,   26,
   27,   28,   29,   30,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   44,    0,    0,    0,    0,
    0,   47,   48,   49,    0,    0,   52,   53,   54,   55,
   56,   57,   58,   59,   60,   61,   62,   63,    0,    0,
   64,   65,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   66,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   67,   68,    0,    8,  124,   10,
    0,   69,    0,   70,   71,   72,    0,   14,    0,   73,
    0,   74,    0,   19,   20,   75,   21,   22,   23,   24,
   25,   26,   27,   28,   29,   30,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   47,   48,   49,    0,    0,   52,   53,
   54,   55,   56,   57,   58,   59,   60,   61,   62,   63,
    0,    0,   64,   65,    0,    0,    0,    0,    8,  124,
   10,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   15,   16,   17,   18,   19,   20,  176,   21,   22,   23,
   24,   25,   26,   27,   28,   29,   30,  177,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   73,    0,   74,    8,  124,   10,   75,    0,    0,
    0,    0,    0,    0,    0,   48,  175,    8,  124,   10,
   19,   20,    0,   21,   22,   23,   24,   25,   26,   27,
   28,   29,   30,   19,   20,    0,   21,   22,   23,   24,
   25,   26,   27,   28,   29,   30,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  176,    0,    0,
  412,  413,  175,    0,    0,    0,    0,    0,  177,    0,
    0,    0,    0,    0,   48,  175,  279,  279,  279,    0,
  279,    0,  279,    0,    0,    0,    0,    0,  178,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  176,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  177,    0,  176,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  177,    0,    0,
    0,   40,   40,   40,  178,   40,    0,   40,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  178,  279,  279,
  279,  279,  279,  279,  279,  279,  279,  279,  279,  279,
  279,  279,  279,  279,  279,  279,  279,  279,  279,  279,
  279,  279,  279,  279,  279,  279,  279,  279,  279,  279,
  279,  279,  279,  279,  279,    0,    0,  279,    0,    0,
  279,  279,  455,  192,    0,  279,  279,  279,  279,  279,
    0,  279,  279,   40,   40,   40,   40,   40,   40,   40,
   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
   40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
    0,    0,   40,    0,    0,   40,   40,    0,    0,    0,
   40,   40,   40,   40,   40,    0,   40,   40,  302,  302,
  302,    0,  302,    0,  302,    0,  193,  194,  195,  196,
  197,  198,  199,  200,  201,  202,  203,  204,  205,  206,
    0,  207,  208,  209,  210,  211,  212,  213,  214,  215,
  216,  217,  218,  219,  220,  221,  222,  223,  224,  225,
    0,    0,  226,    0,    0,  227,  228,    0,    0,    0,
    0,    0,    0,  287,  287,  287,    0,  287,    0,  287,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  302,  302,  302,  302,  302,  302,  302,  302,  302,  302,
  302,  302,  302,  302,  302,  302,  302,  302,  302,  302,
  302,  302,  302,  302,  302,  302,  302,  302,  302,  302,
  302,  302,  302,  302,  302,  302,  302,    0,    0,  302,
    0,    0,  302,  302,    0,  192,    0,  302,  302,  302,
  302,  302,    0,  302,  302,  287,  287,  287,  287,  287,
  287,  287,  287,  287,  287,  287,  287,  287,  287,  287,
  287,  287,  287,  287,  287,  287,  287,  287,  287,  287,
  287,  287,  287,  287,  287,  287,  287,  287,  287,  287,
  287,  287,    0,    0,  287,    0,    0,  287,  287,    0,
    0,    0,  287,  287,  287,  287,  287,    0,  287,  287,
  198,  198,  198,    0,  198,    0,  198,  259,  193,  194,
  195,  196,  197,  198,  199,  200,  201,  202,  203,  204,
  205,  206,    0,  207,  208,  209,  210,  211,  212,  213,
  214,  215,  216,  217,  218,  219,  220,  221,  222,  223,
  224,  225,    0,    0,  226,    0,    0,  227,  228,    0,
    0,    0,    0,    0,    0,  267,  267,  267,    0,  267,
    0,  267,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  198,  198,  198,  198,  198,  198,  198,  198,
  198,  198,  198,  198,  198,  198,  198,  198,  198,  198,
  198,  198,  198,  198,  198,  198,  198,  198,  198,  198,
  198,  198,  198,  198,  198,  198,  198,  198,  198,    0,
    0,  198,    0,    0,  198,  198,    0,  202,  202,  202,
  198,  202,    0,  202,    0,    0,  198,  267,  267,  267,
  267,  267,  267,  267,  267,  267,  267,  267,  267,  267,
  267,  267,  267,  267,  267,  267,  267,  267,  267,  267,
  267,  267,  267,  267,  267,  267,  267,  267,  267,  267,
  267,  267,  267,  267,    0,    0,  267,    0,    0,  267,
  267,    0,  203,  203,  203,  267,  203,    0,  203,    0,
    0,  267,    0,    0,    0,    0,    0,    0,    0,  202,
  202,  202,  202,  202,  202,  202,  202,  202,  202,  202,
  202,  202,  202,  202,  202,  202,  202,  202,  202,  202,
  202,  202,  202,  202,  202,  202,  202,  202,  202,  202,
  202,  202,  202,  202,  202,  202,    0,  201,  201,  201,
    0,  201,    0,  201,    0,    0,    0,  202,    0,    0,
    0,    0,    0,  202,  203,  203,  203,  203,  203,  203,
  203,  203,  203,  203,  203,  203,  203,  203,  203,  203,
  203,  203,  203,  203,  203,  203,  203,  203,  203,  203,
  203,  203,  203,  203,  203,  203,  203,  203,  203,  203,
  203,    0,  205,  205,  205,    0,  205,    0,  205,    0,
    0,    0,  203,    0,    0,    0,    0,    0,  203,  201,
  201,  201,  201,  201,  201,  201,  201,  201,  201,  201,
  201,  201,  201,  201,  201,  201,  201,  201,  201,  201,
  201,  201,  201,  201,  201,  201,  201,  201,  201,  201,
  201,  201,  201,  201,  201,  201,    0,  204,  204,  204,
    0,  204,    0,  204,    0,    0,    0,  201,    0,    0,
    0,    0,    0,  201,  205,  205,  205,  205,  205,  205,
  205,  205,  205,  205,  205,  205,  205,  205,  205,  205,
  205,  205,  205,  205,  205,  205,  205,  205,  205,  205,
  205,  205,  205,  205,  205,  205,  205,  205,  205,  205,
  205,    0,  207,  207,  207,    0,  207,    0,  207,    0,
    0,    0,  205,    0,    0,    0,    0,    0,  205,  204,
  204,  204,  204,  204,  204,  204,  204,  204,  204,  204,
  204,  204,  204,  204,  204,  204,  204,  204,  204,  204,
  204,  204,  204,  204,  204,  204,  204,  204,  204,  204,
  204,  204,  204,  204,  204,  204,    0,  209,  209,  209,
    0,  209,    0,  209,    0,    0,    0,  204,    0,    0,
    0,    0,    0,  204,  207,  207,  207,  207,  207,  207,
  207,  207,  207,  207,  207,  207,  207,  207,  207,  207,
  207,  207,  207,  207,  207,  207,  207,  207,  207,  207,
  207,  207,  207,  207,  207,  207,  207,  207,  207,  207,
  207,    0,  217,  217,  217,    0,  217,    0,  217,    0,
    0,    0,  207,    0,    0,    0,    0,    0,  207,  209,
  209,  209,  209,  209,  209,  209,  209,  209,  209,  209,
  209,  209,  209,  209,  209,  209,  209,  209,  209,  209,
  209,  209,  209,  209,  209,  209,  209,  209,  209,  209,
  209,  209,  209,  209,  209,  209,    0,  214,  214,  214,
    0,  214,    0,  214,    0,    0,    0,  209,    0,    0,
    0,    0,    0,  209,  217,  217,  217,  217,  217,  217,
  217,  217,  217,  217,  217,  217,  217,  217,  217,  217,
  217,  217,  217,  217,  217,  217,  217,  217,  217,  217,
  217,  217,  217,  217,  217,  217,  217,  217,  217,  217,
    0,    0,  215,  215,  215,    0,  215,    0,  215,    0,
    0,    0,  217,    0,    0,    0,    0,    0,  217,  214,
  214,  214,  214,  214,  214,  214,  214,  214,  214,  214,
  214,  214,  214,  214,  214,  214,  214,  214,  214,  214,
  214,  214,  214,  214,  214,  214,  214,  214,  214,  214,
  214,  214,  214,  214,  214,    0,    0,  216,  216,  216,
    0,  216,    0,  216,    0,    0,    0,  214,    0,    0,
    0,    0,    0,  214,  215,  215,  215,  215,  215,  215,
  215,  215,  215,  215,  215,  215,  215,  215,  215,  215,
  215,  215,  215,  215,  215,  215,  215,  215,  215,  215,
  215,  215,  215,  215,  215,  215,  215,  215,  215,  215,
    0,    0,  218,  218,  218,    0,  218,    0,  218,    0,
    0,    0,  215,    0,    0,    0,    0,    0,  215,  216,
  216,  216,  216,  216,  216,  216,  216,  216,  216,  216,
  216,  216,  216,  216,  216,  216,  216,  216,  216,  216,
  216,  216,  216,  216,  216,  216,  216,  216,  216,  216,
  216,  216,  216,  216,  216,    0,    0,  212,  212,    0,
    0,  212,    0,  212,    0,    0,    0,  216,    0,    0,
    0,    0,    0,  216,  218,  218,  218,  218,  218,  218,
  218,  218,  218,  218,  218,  218,  218,  218,  218,  218,
  218,  218,  218,  218,  218,  218,  218,  218,  218,  218,
  218,  218,  218,  218,  218,  218,  218,  218,  218,  218,
    0,    0,  213,  213,    0,    0,  213,    0,  213,    0,
    0,    0,  218,    0,    0,    0,    0,    0,  218,  212,
  212,  212,  212,  212,  212,  212,  212,  212,  212,  212,
  212,  212,  212,  212,  212,  212,  212,  212,  212,  212,
  212,  212,  212,  212,  212,  212,  212,  212,  212,  212,
  212,  212,    0,    0,    0,    0,    0,  219,  219,    0,
    0,  219,    0,  219,    0,    0,    0,  212,    0,    0,
    0,    0,    0,  212,  213,  213,  213,  213,  213,  213,
  213,  213,  213,  213,  213,  213,  213,  213,  213,  213,
  213,  213,  213,  213,  213,  213,  213,  213,  213,  213,
  213,  213,  213,  213,  213,  213,  213,    0,    0,    0,
    0,    0,  220,  220,    0,    0,  220,    0,  220,    0,
    0,    0,  213,    0,    0,    0,    0,    0,  213,  219,
  219,  219,  219,  219,  219,  219,  219,  219,  219,  219,
  219,  219,  219,  219,  219,  219,  219,  219,  219,  219,
  219,  219,  219,  219,  219,  219,  219,  219,  219,  219,
    0,    0,    0,    0,    0,    0,    0,  230,  230,    0,
    0,  230,    0,  230,    0,    0,    0,  219,    0,    0,
    0,    0,    0,  219,  220,  220,  220,  220,  220,  220,
  220,  220,  220,  220,  220,  220,  220,  220,  220,  220,
  220,  220,  220,  220,  220,  220,  220,  220,  220,  220,
  220,  220,  220,  220,  220,    0,    0,    0,    0,    0,
    0,    0,  231,  231,    0,    0,  231,    0,  231,    0,
    0,    0,  220,    0,    0,    0,    0,    0,  220,  230,
  230,  230,  230,  230,  230,  230,  230,  230,  230,  230,
  230,  230,  230,  230,  230,  230,  230,  230,  230,  230,
  230,  230,  230,  230,  230,  230,  230,  230,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  232,  232,    0,
    0,  232,    0,  232,    0,    0,    0,  230,    0,    0,
    0,    0,    0,  230,  231,  231,  231,  231,  231,  231,
  231,  231,  231,  231,  231,  231,  231,  231,  231,  231,
  231,  231,  231,  231,  231,  231,  231,  231,  231,  231,
  231,  231,  231,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  227,  227,    0,    0,  227,    0,  227,    0,
    0,    0,  231,    0,    0,    0,    0,    0,  231,  232,
  232,  232,  232,  232,  232,  232,  232,  232,  232,  232,
  232,  232,  232,  232,  232,  232,  232,  232,  232,  232,
  232,  232,  232,  232,  232,  232,  232,  232,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  228,  228,    0,
    0,  228,    0,  228,    0,    0,    0,  232,    0,    0,
    0,    0,    0,  232,  227,  227,  227,  227,  227,  227,
  227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
  227,  227,  227,  227,  227,  227,  227,  227,  227,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  223,  223,    0,    0,  223,    0,  223,    0,
    0,    0,  227,    0,    0,    0,    0,    0,  227,  228,
  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
  228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
  228,  228,  228,  228,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  222,  222,    0,
    0,  222,    0,  222,    0,    0,    0,  228,    0,    0,
    0,    0,    0,  228,  223,  223,  223,  223,  223,  223,
  223,  223,  223,  223,  223,  223,  223,  223,  223,  223,
  223,  223,  223,  223,  223,  223,  223,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   39,    0,   39,    0,   39,    0,    0,    0,    0,    0,
    0,    0,  223,    0,    0,    0,    0,    0,  223,  222,
  222,  222,  222,  222,  222,  222,  222,  222,  222,  222,
  222,  222,  222,  222,  222,  222,  222,  222,  222,  222,
  222,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  266,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  222,    0,    0,
   39,    0,   39,  222,   39,   39,   39,   39,   39,   39,
   39,   39,   39,   39,   39,   39,   39,    0,   39,   39,
   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,
   39,   39,   39,   39,   39,   39,   39,  192,    0,   39,
    0,  386,   39,   39,    0,    0,    0,   39,    0,   39,
   39,   39,    0,   39,  266,  266,  266,  266,  266,  266,
  266,  266,  266,  266,  266,  266,  266,  266,  266,    0,
  266,  266,  266,  266,  266,  266,  266,  266,  266,  266,
  266,  266,  266,  266,  266,  266,  266,  266,  266,  192,
    0,  266,    0,    0,  266,  266,    0,    0,    0,  266,
    0,  266,  266,  266,    0,  266,    0,    0,    0,  427,
  193,  194,  195,  196,  197,  198,  199,  200,  201,  202,
  203,  204,  205,  206,    0,  207,  208,  209,  210,  211,
  212,  213,  214,  215,  216,  217,  218,  219,  220,  221,
  222,  223,  224,  225,  192,    0,  226,    0,    0,  227,
  228,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  380,  193,  194,  195,  196,  197,  198,  199,  200,
  201,  202,  203,  204,  205,  206,    0,  207,  208,  209,
  210,  211,  212,  213,  214,  215,  216,  217,  218,  219,
  220,  221,  222,  223,  224,  225,  192,    0,  226,    0,
    0,  227,  228,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  193,  194,  195,
  196,  197,  198,  199,  200,  201,  202,  203,  204,  205,
  206,  424,  207,  208,  209,  210,  211,  212,  213,  214,
  215,  216,  217,  218,  219,  220,  221,  222,  223,  224,
  225,  192,    0,  226,    0,    0,  227,  228,    0,    0,
    0,    0,    0,    0,    0,    0,  511,    0,    0,  193,
  194,  195,  196,  197,  198,  199,  200,  201,  202,  203,
  204,  205,  206,    0,  207,  208,  209,  210,  211,  212,
  213,  214,  215,  216,  217,  218,  219,  220,  221,  222,
  223,  224,  225,    0,    0,  226,    0,    0,  227,  228,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  193,  194,  195,  196,  197,  198,
  199,  200,  201,  202,  203,  204,  205,  206,    0,  207,
  208,  209,  210,  211,  212,  213,  214,  215,  216,  217,
  218,  219,  220,  221,  222,  223,  224,  225,    0,    0,
  226,    0,    0,  227,  228,
};
short yycheck[] = {                                       6,
   75,   43,   43,   32,    6,  512,  170,    6,  266,   80,
  339,   43,  237,  174,  316,  101,  102,  181,  290,  239,
  271,  303,  242,  187,  290,  290,   78,  269,  289,  265,
  266,  290,  290,  339,   75,  265,   43,  171,  266,  271,
  572,   93,  266,   75,   43,  116,  268,   44,  271,  265,
  266,  290,  271,   50,  265,  266,  385,  589,  268,   66,
  290,   68,  265,  266,  269,  347,  271,   82,   75,   66,
   67,   68,   69,   70,   71,   72,   75,  163,  269,  385,
  268,   88,  134,   90,  316,  137,  316,  162,  339,  391,
   97,  268,  316,  317,  101,  102,  230,  339,  266,  233,
  339,  268,  117,  118,  119,  120,  121,  339,  265,  266,
  338,  618,  416,  385,  266,  339,  339,  132,  340,  126,
  339,  382,  338,  339,  131,  391,  391,  338,  339,  131,
  265,  266,  391,  391,  339,  338,  339,  393,  354,  359,
  391,  265,  266,  354,  151,  309,  385,  385,  339,  190,
  314,  354,  316,  168,  151,  391,  163,  266,  190,  391,
  384,  265,  266,  340,  171,  172,  390,  290,  498,  176,
  177,  178,  289,  340,  171,  391,  480,  265,  266,  186,
  391,  338,  339,  190,  265,  266,  376,  160,  391,  379,
  380,  190,  316,  317,  318,  192,  526,  354,  459,  316,
  386,  265,  290,  338,  339,  340,  269,  265,  266,  206,
  207,  208,  209,  210,  211,  212,  213,  214,  215,  216,
  217,  218,  219,  220,  221,  222,  223,  224,  225,  244,
  237,  513,  229,  230,  391,  339,  233,  234,  235,  400,
  401,  269,  403,  404,  251,  506,  380,  318,  265,  251,
  511,  248,  316,  316,   90,  419,  268,  338,  273,  266,
  266,  266,  259,  269,  289,  269,  269,  319,  271,  266,
  384,  297,  289,  354,  265,  266,  390,  265,  266,  265,
  338,  269,  279,  271,  313,  290,  290,  290,  316,  265,
  126,  316,  289,  256,  306,  271,  354,  317,  318,  316,
  525,  387,  265,  289,  290,  312,  386,  316,  317,  318,
  391,  391,  398,  289,  290,  385,  386,  291,  265,  293,
  294,  268,  363,  364,  316,  366,  367,  369,  369,   32,
  316,  363,  364,  391,  366,  367,  316,  369,  318,  339,
  316,  317,  289,  290,  267,  431,  432,  385,  386,  337,
  338,  339,  316,  269,  318,  316,  363,  364,  268,  366,
  367,  316,  369,  339,  363,  364,  354,  366,  367,  316,
  369,  316,  317,  598,  290,  291,  390,  293,  294,  266,
  387,  297,  339,  380,  609,   88,  316,   90,  318,  614,
  265,  398,  433,  390,  311,  312,  411,  385,  101,  102,
  407,  433,  391,  391,  290,  291,  631,  293,  294,  385,
  386,  297,  391,  442,  289,  391,  291,  265,  293,  294,
  311,  312,  297,  126,  431,  432,  433,  424,  393,  269,
  427,  271,  316,  448,  433,  450,  451,  384,  385,  386,
  464,  316,  466,  390,  391,  460,  390,  370,  371,  372,
  373,  265,  266,  376,  495,  496,  379,  380,  500,  500,
  163,  340,  459,  495,  496,  316,  317,  318,  500,  172,
  316,  502,  503,  176,  177,  178,    0,  265,  316,  316,
  266,  454,  269,  186,  499,  269,  493,  494,  495,  496,
  497,  493,  494,  500,  269,  497,  495,  496,  471,  574,
  339,  500,  269,  290,  291,  373,  293,  294,  376,  506,
  297,  379,  380,  390,  511,  512,  390,  514,  525,  321,
  322,  323,  324,  325,  311,  312,  390,  390,  603,  570,
  479,  390,  390,  266,  483,  385,  393,  316,  570,  536,
  316,  390,  539,  339,  541,  339,  391,  520,  390,  267,
  385,  385,  316,  316,  527,  528,  385,  564,  531,  532,
  391,  391,  564,  570,  537,  385,  391,  540,  575,  542,
  269,  570,  587,  575,  523,  266,  384,  267,  386,  387,
  388,  266,  390,  390,  303,  391,  314,  265,  265,  562,
  391,  598,  370,  391,  567,  268,  391,  271,  391,  266,
  339,  266,  609,    0,  265,  390,  266,  614,  391,  312,
  313,  391,  339,  267,  266,  266,  565,  566,  591,  568,
  569,  618,  391,  391,  631,  340,  339,  339,  601,  602,
  391,  390,  266,  268,  391,  584,  585,  586,  268,  268,
  268,  268,  391,  269,  316,  594,  271,  620,  366,  367,
  368,  369,  370,  371,  372,  373,  269,  316,  376,  632,
  302,  379,  380,  612,  339,  391,  269,  266,  617,  359,
  360,  361,  362,  363,  364,  365,  366,  367,  368,  369,
  370,  371,  372,  373,  387,  269,  376,  385,  354,  379,
  380,  354,  354,  316,  316,  398,  269,  354,  290,  297,
  265,  290,  265,  385,  407,  391,  360,  361,  362,  363,
  364,  365,  366,  367,  368,  369,  370,  371,  372,  373,
  316,  385,  376,  266,    0,  379,  380,  267,  431,  432,
  316,  269,  256,  391,  269,  269,  260,  261,  262,  442,
  269,  265,  266,  267,  268,  269,  270,  385,  272,  273,
  274,  275,  276,  277,  269,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,  292,  293,
  294,  354,  261,  297,  298,  299,  300,  301,  302,  303,
  304,  305,  306,  307,  308,  309,  310,  311,  312,  313,
  399,  315,  316,  317,  318,  319,  320,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,  331,  332,  590,
  582,  335,  336,  353,  171,  355,  356,  357,  358,  359,
  360,  361,  362,  363,  364,  365,  366,  367,  368,  369,
  370,  371,  372,  373,  431,  359,  376,  573,  397,  379,
  380,  432,  478,  407,  398,  369,  370,  534,  387,  496,
  248,  267,  376,  259,  378,  379,  380,  427,  453,  256,
  384,   14,  386,  260,  261,  262,  390,  511,  265,  266,
  267,  268,  269,  270,  514,  272,  273,  274,  275,  276,
  277,  501,  279,  280,  281,  282,  283,  284,  285,  286,
  287,  288,  289,  290,  291,  292,  293,  294,  516,  260,
  297,  298,  299,  300,  301,   -1,  303,  304,  305,  306,
  307,  308,  309,  310,  311,  312,  313,  314,  315,  316,
  317,  318,  319,  320,  321,  322,  323,  324,  325,  326,
  327,  328,  329,  330,  331,  332,   -1,   -1,  335,  336,
  356,  357,  358,  359,  360,  361,  362,  363,  364,  365,
  366,  367,  368,  369,  370,  371,  372,  373,   -1,   -1,
  376,   -1,  359,  379,  380,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  369,  370,   -1,  267,   -1,   -1,   -1,  376,
  256,  378,  379,  380,  260,  261,  262,  384,   -1,  386,
  266,  267,  268,  390,  270,   -1,  272,  273,  274,  275,
  276,  277,   -1,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,   -1,
   -1,  297,  298,  299,  300,  301,   -1,  303,  304,  305,
  306,  307,  308,  309,  310,   -1,   -1,  313,   -1,  315,
  316,  317,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,  331,  332,   -1,   31,  335,
  336,  267,   -1,   36,   37,   38,   39,   40,   -1,   -1,
   -1,   -1,   45,   46,   -1,   -1,  368,  369,  370,  371,
  372,  373,   -1,  359,  376,   -1,   -1,  379,  380,   -1,
   -1,   -1,   -1,  369,  370,   -1,   -1,   -1,   -1,   -1,
  376,   -1,  378,  379,  380,  265,  266,   -1,  384,  269,
  386,  271,   -1,   -1,  390,   88,   -1,   90,  260,  261,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  103,   -1,   -1,   -1,  276,  277,   -1,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,   -1,   -1,   -1,
   -1,   -1,   -1,  126,   -1,   -1,  362,  363,  364,  365,
  366,  367,  368,  369,  370,  371,  372,  373,   -1,   -1,
  376,   -1,   -1,  379,  380,  317,  318,  337,  338,  339,
  340,  341,  342,  343,  344,  345,  346,  347,  348,  349,
  350,  351,  352,  353,  354,  355,  356,  357,  358,  359,
  360,  361,  362,  363,  364,  365,   -1,  180,   -1,  182,
  183,  184,  185,  186,   -1,  188,   -1,  359,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  385,   -1,   -1,  370,   -1,
   -1,  391,  267,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  260,  261,  262,   -1,  390,  391,
  266,  267,  268,   -1,  270,   -1,  272,  273,  274,  275,
  276,  277,   -1,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,   -1,
   -1,  297,  298,  299,  300,  301,   -1,  303,  304,  305,
  306,  307,  308,  309,  310,   -1,   -1,  313,   -1,  315,
  316,  317,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,  331,  332,   -1,   -1,  335,
  336,   -1,  357,  358,  359,  360,  361,  362,  363,  364,
  365,  366,  367,  368,  369,  370,  371,  372,  373,   -1,
   -1,  376,  315,  359,  379,  380,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  369,  370,   -1,  267,   -1,   -1,   -1,
  376,   -1,  378,  379,  380,  260,  261,  262,  384,   -1,
  386,  266,  267,  268,  390,  270,   -1,  272,  273,  274,
  275,  276,  277,   -1,  279,  280,  281,  282,  283,  284,
  285,  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,
   -1,   -1,   -1,  298,  299,  300,  301,   -1,  303,  304,
  305,  306,  307,  308,  309,  310,   -1,   -1,  313,   -1,
  315,  316,  317,  318,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,  331,  332,   -1,   -1,
  335,  336,   -1,   -1,   -1,   -1,   -1,  358,  359,  360,
  361,  362,  363,  364,  365,  366,  367,  368,  369,  370,
  371,  372,  373,   -1,  359,  376,   -1,   -1,  379,  380,
   -1,   -1,   -1,   -1,  369,  370,   -1,   -1,   -1,   -1,
   -1,  376,   -1,  378,  379,  380,  260,  261,  262,  384,
   -1,  386,  266,  267,  268,  390,  270,   -1,  272,  273,
  274,  275,  276,  277,   -1,  279,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1,
   -1,   -1,   -1,   -1,  298,  299,  300,  301,   -1,  303,
  304,  305,  306,  307,  308,  309,  310,   -1,   -1,  313,
   -1,  315,  316,  317,  318,   -1,   -1,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,  331,  332,   -1,
   -1,  335,  336,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  359,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  369,  370,   -1,   -1,   -1,
   -1,   -1,  376,   -1,  378,  379,  380,  260,  261,  262,
  384,   -1,  386,   -1,  267,  268,  390,  270,   -1,  272,
  273,  274,  275,  276,  277,   -1,  279,  280,  281,  282,
  283,  284,  285,  286,  287,  288,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  298,  299,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  310,   -1,   -1,
   -1,   -1,   -1,  316,  317,  318,   -1,   -1,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,  331,  332,
   -1,   -1,  335,  336,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  359,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  369,  370,   -1,   -1,
   -1,   -1,   -1,  376,   -1,  378,  379,  380,  260,  261,
  262,  384,   -1,  386,   -1,  267,   -1,  390,  270,   -1,
  272,  273,  274,  275,  276,  277,   -1,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  298,  299,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  310,   -1,
   -1,   -1,   -1,   -1,  316,  317,  318,   -1,   -1,  321,
  322,  323,  324,  325,  326,  327,  328,  329,  330,  331,
  332,   -1,   -1,  335,  336,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  265,  266,   -1,   -1,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  359,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  369,  370,   -1,
   -1,   -1,   -1,   -1,  376,   -1,  378,  379,  380,   -1,
   -1,   -1,  384,   -1,  386,  260,  261,  262,  390,   -1,
   -1,   -1,  267,  268,  269,  270,   -1,   -1,   -1,   -1,
   -1,  276,  277,   -1,  279,  280,  281,  282,  283,  284,
  285,  286,  287,  288,   -1,  337,  338,  339,  340,  341,
  342,  343,  344,  345,  346,  347,  348,  349,  350,  351,
  352,  353,  354,  355,  356,  310,   -1,   -1,   -1,   -1,
   -1,  316,  317,  318,   -1,   -1,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,  331,  332,   -1,   -1,
  335,  336,  337,  385,   -1,   -1,   -1,   -1,   -1,  391,
   -1,   -1,  265,  266,   -1,   -1,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  359,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  369,  370,   -1,   -1,   -1,   -1,
   -1,  376,   -1,  378,  379,  380,   -1,   -1,   -1,  384,
   -1,  386,  260,  261,  262,  390,   -1,   -1,   -1,  267,
   -1,  269,  270,   -1,   -1,   -1,   -1,   -1,  276,  277,
   -1,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,   -1,   -1,   -1,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
   -1,  354,  310,   -1,   -1,   -1,   -1,   -1,  316,  317,
  318,   -1,   -1,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  330,  331,  332,   -1,   -1,  335,  336,  337,
   -1,   -1,  385,   -1,   -1,   -1,  265,  266,  391,   -1,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  359,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  369,  370,   -1,   -1,   -1,   -1,   -1,  376,   -1,
  378,  379,  380,   -1,   -1,   -1,  384,   -1,  386,  260,
  261,  262,  390,   -1,   -1,   -1,  267,  268,  269,  270,
   -1,   -1,   -1,   -1,   -1,  276,  277,   -1,  279,  280,
  281,  282,  283,  284,  285,  286,  287,  288,  337,  338,
  339,  340,  341,  342,  343,  344,  345,  346,  347,  348,
  349,  350,  351,  352,  353,  354,  355,  356,  357,  310,
   -1,   -1,   -1,   -1,   -1,  316,  317,  318,   -1,   -1,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
  331,  332,   -1,   -1,  335,  336,  385,  265,  266,   -1,
   -1,  269,  391,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  359,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  369,  370,
   -1,   -1,   -1,   -1,   -1,  376,   -1,  378,  379,  380,
  260,  261,  262,  384,   -1,  386,   -1,  267,   -1,  390,
  270,   -1,   -1,   -1,   -1,   -1,  276,  277,   -1,  279,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,   -1,   -1,
  310,   -1,   -1,   -1,   -1,   -1,  316,  317,  318,   -1,
   -1,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,  331,  332,   -1,   -1,  335,  336,  385,  265,  266,
   -1,   -1,  269,  391,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  359,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  369,
  370,   -1,   -1,   -1,   -1,   -1,  376,   -1,  378,  379,
  380,  260,  261,  262,  384,  385,  386,   -1,  267,   -1,
  390,  270,  271,   -1,   -1,   -1,   -1,  276,  277,   -1,
  279,  280,  281,  282,  283,  284,  285,  286,  287,  288,
  337,  338,  339,  340,  341,  342,  343,  344,  345,  346,
  347,  348,  349,  350,  351,  352,   -1,  354,   -1,   -1,
   -1,  310,   -1,   -1,   -1,   -1,   -1,  316,  317,  318,
   -1,   -1,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,  331,  332,   -1,   -1,  335,  336,  385,   -1,
   -1,   -1,   -1,   -1,  391,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  359,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  369,  370,   -1,   -1,   -1,   -1,   -1,  376,   -1,  378,
  379,  380,  260,  261,  262,  384,   -1,  386,   -1,  267,
  268,  390,  270,   -1,   -1,   -1,   -1,   -1,  276,  277,
   -1,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  310,   -1,   -1,   -1,   -1,   -1,  316,  317,
  318,   -1,   -1,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  330,  331,  332,   -1,   -1,  335,  336,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  359,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  369,  370,   -1,   -1,   -1,   -1,   -1,  376,   -1,
  378,  379,  380,  260,  261,  262,  384,   -1,  386,   -1,
  267,   -1,  390,  270,   -1,   -1,   -1,   -1,   -1,  276,
  277,   -1,  279,  280,  281,  282,  283,  284,  285,  286,
  287,  288,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  310,   -1,   -1,   -1,   -1,   -1,  316,
  317,  318,   -1,   -1,  321,  322,  323,  324,  325,  326,
  327,  328,  329,  330,  331,  332,   -1,   -1,  335,  336,
  337,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  359,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  369,  370,   -1,   -1,   -1,   -1,   -1,  376,
   -1,  378,  379,  380,  260,  261,  262,  384,   -1,  386,
   -1,  267,  268,  390,  270,   -1,   -1,   -1,   -1,   -1,
  276,  277,   -1,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  310,   -1,   -1,   -1,   -1,   -1,
  316,  317,  318,   -1,   -1,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,  331,  332,   -1,   -1,  335,
  336,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  359,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  369,  370,   -1,   -1,   -1,   -1,   -1,
  376,   -1,  378,  379,  380,  260,  261,  262,  384,   -1,
  386,   -1,  267,   -1,  390,  270,   -1,   -1,   -1,   -1,
   -1,  276,  277,   -1,  279,  280,  281,  282,  283,  284,
  285,  286,  287,  288,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  310,   -1,   -1,   -1,   -1,
   -1,  316,  317,  318,   -1,   -1,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,  331,  332,   -1,   -1,
  335,  336,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  359,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  369,  370,   -1,  260,  261,  262,
   -1,  376,   -1,  378,  379,  380,   -1,  270,   -1,  384,
   -1,  386,   -1,  276,  277,  390,  279,  280,  281,  282,
  283,  284,  285,  286,  287,  288,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  316,  317,  318,   -1,   -1,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,  331,  332,
   -1,   -1,  335,  336,   -1,   -1,   -1,   -1,  260,  261,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  272,  273,  274,  275,  276,  277,  359,  279,  280,  281,
  282,  283,  284,  285,  286,  287,  288,  370,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  384,   -1,  386,  260,  261,  262,  390,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  317,  318,  260,  261,  262,
  276,  277,   -1,  279,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  276,  277,   -1,  279,  280,  281,  282,
  283,  284,  285,  286,  287,  288,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  359,   -1,   -1,
  316,  317,  318,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,   -1,   -1,  317,  318,  265,  266,  267,   -1,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  390,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  359,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,  359,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  265,  266,  267,  390,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  390,  337,  338,
  339,  340,  341,  342,  343,  344,  345,  346,  347,  348,
  349,  350,  351,  352,  353,  354,  355,  356,  357,  358,
  359,  360,  361,  362,  363,  364,  365,  366,  367,  368,
  369,  370,  371,  372,  373,   -1,   -1,  376,   -1,   -1,
  379,  380,  266,  267,   -1,  384,  385,  386,  387,  388,
   -1,  390,  391,  337,  338,  339,  340,  341,  342,  343,
  344,  345,  346,  347,  348,  349,  350,  351,  352,  353,
  354,  355,  356,  357,  358,  359,  360,  361,  362,  363,
  364,  365,  366,  367,  368,  369,  370,  371,  372,  373,
   -1,   -1,  376,   -1,   -1,  379,  380,   -1,   -1,   -1,
  384,  385,  386,  387,  388,   -1,  390,  391,  265,  266,
  267,   -1,  269,   -1,  271,   -1,  340,  341,  342,  343,
  344,  345,  346,  347,  348,  349,  350,  351,  352,  353,
   -1,  355,  356,  357,  358,  359,  360,  361,  362,  363,
  364,  365,  366,  367,  368,  369,  370,  371,  372,  373,
   -1,   -1,  376,   -1,   -1,  379,  380,   -1,   -1,   -1,
   -1,   -1,   -1,  265,  266,  267,   -1,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  337,  338,  339,  340,  341,  342,  343,  344,  345,  346,
  347,  348,  349,  350,  351,  352,  353,  354,  355,  356,
  357,  358,  359,  360,  361,  362,  363,  364,  365,  366,
  367,  368,  369,  370,  371,  372,  373,   -1,   -1,  376,
   -1,   -1,  379,  380,   -1,  267,   -1,  384,  385,  386,
  387,  388,   -1,  390,  391,  337,  338,  339,  340,  341,
  342,  343,  344,  345,  346,  347,  348,  349,  350,  351,
  352,  353,  354,  355,  356,  357,  358,  359,  360,  361,
  362,  363,  364,  365,  366,  367,  368,  369,  370,  371,
  372,  373,   -1,   -1,  376,   -1,   -1,  379,  380,   -1,
   -1,   -1,  384,  385,  386,  387,  388,   -1,  390,  391,
  265,  266,  267,   -1,  269,   -1,  271,  339,  340,  341,
  342,  343,  344,  345,  346,  347,  348,  349,  350,  351,
  352,  353,   -1,  355,  356,  357,  358,  359,  360,  361,
  362,  363,  364,  365,  366,  367,  368,  369,  370,  371,
  372,  373,   -1,   -1,  376,   -1,   -1,  379,  380,   -1,
   -1,   -1,   -1,   -1,   -1,  265,  266,  267,   -1,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  337,  338,  339,  340,  341,  342,  343,  344,
  345,  346,  347,  348,  349,  350,  351,  352,  353,  354,
  355,  356,  357,  358,  359,  360,  361,  362,  363,  364,
  365,  366,  367,  368,  369,  370,  371,  372,  373,   -1,
   -1,  376,   -1,   -1,  379,  380,   -1,  265,  266,  267,
  385,  269,   -1,  271,   -1,   -1,  391,  337,  338,  339,
  340,  341,  342,  343,  344,  345,  346,  347,  348,  349,
  350,  351,  352,  353,  354,  355,  356,  357,  358,  359,
  360,  361,  362,  363,  364,  365,  366,  367,  368,  369,
  370,  371,  372,  373,   -1,   -1,  376,   -1,   -1,  379,
  380,   -1,  265,  266,  267,  385,  269,   -1,  271,   -1,
   -1,  391,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
  368,  369,  370,  371,  372,  373,   -1,  265,  266,  267,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,  368,  369,  370,  371,  372,
  373,   -1,  265,  266,  267,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
  368,  369,  370,  371,  372,  373,   -1,  265,  266,  267,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,  368,  369,  370,  371,  372,
  373,   -1,  265,  266,  267,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
  368,  369,  370,  371,  372,  373,   -1,  265,  266,  267,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,  368,  369,  370,  371,  372,
  373,   -1,  265,  266,  267,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
  368,  369,  370,  371,  372,  373,   -1,  265,  266,  267,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,  368,  369,  370,  371,  372,
   -1,   -1,  265,  266,  267,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
  368,  369,  370,  371,  372,   -1,   -1,  265,  266,  267,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,  368,  369,  370,  371,  372,
   -1,   -1,  265,  266,  267,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
  368,  369,  370,  371,  372,   -1,   -1,  265,  266,   -1,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,  368,  369,  370,  371,  372,
   -1,   -1,  265,  266,   -1,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
  368,  369,   -1,   -1,   -1,   -1,   -1,  265,  266,   -1,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,  368,  369,   -1,   -1,   -1,
   -1,   -1,  265,  266,   -1,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  265,  266,   -1,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  265,  266,   -1,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  265,  266,   -1,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  265,  266,   -1,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  265,  266,   -1,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  265,  266,   -1,   -1,  269,   -1,  271,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,  359,  360,  361,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  265,  266,   -1,
   -1,  269,   -1,  271,   -1,   -1,   -1,  385,   -1,   -1,
   -1,   -1,   -1,  391,  337,  338,  339,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  267,   -1,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  385,   -1,   -1,   -1,   -1,   -1,  391,  337,
  338,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  267,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  385,   -1,   -1,
  337,   -1,  339,  391,  341,  342,  343,  344,  345,  346,
  347,  348,  349,  350,  351,  352,  353,   -1,  355,  356,
  357,  358,  359,  360,  361,  362,  363,  364,  365,  366,
  367,  368,  369,  370,  371,  372,  373,  267,   -1,  376,
   -1,  271,  379,  380,   -1,   -1,   -1,  384,   -1,  386,
  387,  388,   -1,  390,  339,  340,  341,  342,  343,  344,
  345,  346,  347,  348,  349,  350,  351,  352,  353,   -1,
  355,  356,  357,  358,  359,  360,  361,  362,  363,  364,
  365,  366,  367,  368,  369,  370,  371,  372,  373,  267,
   -1,  376,   -1,   -1,  379,  380,   -1,   -1,   -1,  384,
   -1,  386,  387,  388,   -1,  390,   -1,   -1,   -1,  339,
  340,  341,  342,  343,  344,  345,  346,  347,  348,  349,
  350,  351,  352,  353,   -1,  355,  356,  357,  358,  359,
  360,  361,  362,  363,  364,  365,  366,  367,  368,  369,
  370,  371,  372,  373,  267,   -1,  376,   -1,   -1,  379,
  380,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  339,  340,  341,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,  353,   -1,  355,  356,  357,
  358,  359,  360,  361,  362,  363,  364,  365,  366,  367,
  368,  369,  370,  371,  372,  373,  267,   -1,  376,   -1,
   -1,  379,  380,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  340,  341,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  361,  362,
  363,  364,  365,  366,  367,  368,  369,  370,  371,  372,
  373,  267,   -1,  376,   -1,   -1,  379,  380,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  337,   -1,   -1,  340,
  341,  342,  343,  344,  345,  346,  347,  348,  349,  350,
  351,  352,  353,   -1,  355,  356,  357,  358,  359,  360,
  361,  362,  363,  364,  365,  366,  367,  368,  369,  370,
  371,  372,  373,   -1,   -1,  376,   -1,   -1,  379,  380,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  340,  341,  342,  343,  344,  345,
  346,  347,  348,  349,  350,  351,  352,  353,   -1,  355,
  356,  357,  358,  359,  360,  361,  362,  363,  364,  365,
  366,  367,  368,  369,  370,  371,  372,  373,   -1,   -1,
  376,   -1,   -1,  379,  380,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 393
#if YYDEBUG
char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"VAR","EXPR","ARRAY","STRUCT",
"UNION","ENUM","COMP","HASH","NL","SEMI","MOD","OC","CC","DOLLAR","DOTDOTDOT",
"GLOBAL","AUTO","STATIC","CONST","POLY","INTEGER","NATURAL","RATIONAL","REAL",
"STRING","FILET","MUTEX","SEMAPHORE","CONTINUATION","THREAD","VOID","BOOL",
"FUNCTION","FUNC","EXCEPTION","RAISE","TYPEDEF","IMPORT","NEW","ANONINIT",
"NAMESPACE","PUBLIC","PROTECTED","EXTEND","IF","ELSE","WHILE","DO","FOR",
"SWITCH","BREAK","CONTINUE","RETURNTOK","FORK","CASE","DEFAULT","TRY","CATCH",
"TWIXT","NAME","TYPENAME","NAMESPACENAME","COMMAND","NAMECOMMAND","TEN_NUM",
"OCTAL0_NUM","OCTAL_NUM","BINARY_NUM","HEX_NUM","TEN_FLOAT","OCTAL0_FLOAT",
"OCTAL_FLOAT","BINARY_FLOAT","HEX_FLOAT","CHAR_CONST","STRING_CONST",
"POLY_CONST","THREAD_CONST","VOIDVAL","BOOLVAL","DARROW","POUND","COMMA",
"ASSIGN","ASSIGNPLUS","ASSIGNMINUS","ASSIGNTIMES","ASSIGNDIVIDE","ASSIGNDIV",
"ASSIGNMOD","ASSIGNPOW","ASSIGNSHIFTL","ASSIGNSHIFTR","ASSIGNLXOR","ASSIGNLAND",
"ASSIGNLOR","QUEST","COLON","OR","AND","LOR","LXOR","LAND","EQ","NE","LT","GT",
"LE","GE","SHIFTL","SHIFTR","PLUS","MINUS","TIMES","DIVIDE","DIV","POW",
"UNIONCAST","UMINUS","BANG","FACT","LNOT","INC","DEC","STAR","AMPER","THREADID",
"OS","CS","DOT","ARROW","STAROS","CALL","OP","CP","POINTER","COLONCOLON",
};
char *yyrule[] = {
"$accept : lines",
"lines : lines pcommand",
"lines :",
"pcommand : command",
"pcommand : error reset eend",
"eend : NL",
"eend : SEMI",
"ignorenl :",
"attendnl :",
"reset :",
"not_command :",
"opt_nl : NL",
"opt_nl :",
"opt_semi : SEMI",
"opt_semi :",
"opt_comma : COMMA",
"opt_comma :",
"command : not_command expr reset NL",
"command : not_command expr POUND expr reset NL",
"command : not_command statement reset opt_nl",
"command : not_command COMMAND opt_exprs reset opt_semi NL",
"command : not_command NAMECOMMAND opt_rawnames reset opt_semi NL",
"command : NL",
"opt_rawnames : rawnames",
"opt_rawnames :",
"rawnames : rawname COMMA rawnames",
"rawnames : rawname",
"rawname : rawnamespace rawatom",
"rawatom : NAME",
"rawatom : NAMESPACENAME",
"rawatom : TYPENAME",
"rawnamespace : rawnamespace rawatom COLONCOLON",
"rawnamespace :",
"fulltype : namespace TYPENAME",
"fulltype : TYPENAME",
"fullname : namespace namespacename",
"fullname : namespacename",
"namespace : namespace NAMESPACENAME COLONCOLON",
"namespace : NAMESPACENAME COLONCOLON",
"namespacename : NAME",
"namespacename : NAMESPACENAME",
"namespace_start :",
"namespace_end :",
"block : block_start statements block_end",
"block_start : OC namespace_start",
"block_end : namespace_end CC",
"statements : statement statements",
"statements :",
"statement : IF ignorenl namespace_start OP expr CP statement namespace_end attendnl",
"statement : IF ignorenl namespace_start OP expr CP statement ELSE statement namespace_end attendnl",
"statement : WHILE ignorenl namespace_start OP expr CP statement namespace_end attendnl",
"statement : DO ignorenl namespace_start statement WHILE OP expr CP namespace_end attendnl",
"statement : FOR ignorenl namespace_start OP for_exprs CP statement namespace_end attendnl",
"statement : SWITCH ignorenl namespace_start OP expr CP case_block namespace_end attendnl",
"statement : UNION SWITCH ignorenl namespace_start OP expr CP union_case_block namespace_end attendnl",
"statement : BREAK SEMI",
"statement : CONTINUE SEMI",
"statement : RETURNTOK opt_expr SEMI",
"statement : expr SEMI",
"statement : block",
"statement : SEMI",
"statement : func_decl opt_func_body namespace_end",
"statement : opt_publish EXCEPTION ignorenl NAME namespace_start opt_argdecls namespace_end SEMI attendnl",
"statement : RAISE fullname OP opt_exprs CP SEMI",
"statement : opt_publish TYPEDEF ignorenl typenames SEMI attendnl",
"statement : opt_publish TYPEDEF ignorenl type typenames SEMI attendnl",
"$$1 :",
"statement : publish_extend NAMESPACE ignorenl namespacename $$1 OC statements CC attendnl",
"statement : opt_publish IMPORT ignorenl fullname SEMI attendnl",
"statement : TRY ignorenl statement catches attendnl",
"statement : TWIXT ignorenl namespace_start OP opt_expr SEMI opt_expr CP statement namespace_end attendnl",
"for_exprs : opt_expr SEMI for_exprs",
"for_exprs : opt_expr",
"catches : catches catch",
"catches :",
"catch : CATCH fullname namespace_start args block namespace_end",
"opt_func_body : func_body",
"opt_func_body : SEMI",
"$$2 :",
"func_body : $$2 block_or_expr",
"block_or_expr : block",
"block_or_expr : ASSIGN simpleexpr SEMI",
"case_block : block_start cases block_end",
"cases : case cases",
"cases :",
"case : CASE expr COLON statements",
"case : DEFAULT COLON statements",
"union_case_block : block_start union_cases block_end",
"union_cases : union_case union_cases",
"union_cases :",
"union_case : CASE namespace_start NAME opt_name COLON statements namespace_end",
"union_case : DEFAULT COLON statements",
"opt_name : NAME",
"opt_name :",
"atoms : NAME COMMA atoms",
"atoms : NAME",
"typenames : typename COMMA typenames",
"typenames : typename",
"typename : TYPENAME",
"typename : NAME",
"initnames : name opt_init next_decl initnames",
"initnames : name opt_init",
"name : NAME",
"next_decl : COMMA",
"func_decl : func_name namespace_start opt_argdefines CP",
"func_name : decl NAME OP",
"func_name : decl FUNCTION NAME OP",
"func_name : FUNCTION ignorenl NAME OP",
"opt_init : ASSIGN simpleexpr",
"opt_init : ASSIGN init",
"opt_init :",
"decl : publish class type ignorenl opt_nl",
"decl : class type ignorenl opt_nl",
"decl : publish type ignorenl opt_nl",
"decl : type ignorenl opt_nl",
"decl : publish class ignorenl opt_nl",
"decl : class ignorenl opt_nl",
"decl : publish ignorenl opt_nl",
"subscripts : opt_argdecls subscripts",
"subscripts : OS opt_stars CS subscripts",
"subscripts : OS dotdotdots CS subscripts",
"subscripts : OS dims CS subscripts",
"subscripts : OS type CS subscripts",
"subscripts :",
"type : subtype subscripts",
"type : TIMES type",
"type : LAND type",
"subtype : basetype",
"subtype : STRUCT OC struct_members CC",
"subtype : UNION OC union_members CC",
"subtype : ENUM OC atoms CC",
"subtype : OP type CP",
"subtype : fulltype",
"opt_type : type",
"opt_type :",
"basetype : POLY",
"basetype : INTEGER",
"basetype : RATIONAL",
"basetype : REAL",
"basetype : STRING",
"basetype : FILET",
"basetype : MUTEX",
"basetype : SEMAPHORE",
"basetype : CONTINUATION",
"basetype : THREAD",
"basetype : VOID",
"basetype : BOOL",
"opt_stars : stars",
"opt_stars :",
"stars : stars COMMA TIMES",
"stars : TIMES",
"dotdotdots : dotdotdots COMMA DOTDOTDOT",
"dotdotdots : DOTDOTDOT",
"dims : simpleexpr COMMA dims",
"dims : simpleexpr",
"struct_members : opt_type atoms SEMI struct_members",
"struct_members :",
"union_members : opt_type atoms SEMI union_members",
"union_members :",
"class : GLOBAL",
"class : AUTO",
"class : STATIC",
"class : CONST",
"opt_publish : publish",
"opt_publish :",
"publish : PUBLIC",
"publish : PROTECTED",
"publish_extend : opt_publish",
"publish_extend : EXTEND",
"opt_argdecls : OP argdecls CP",
"opt_argdecls : OP CP",
"argdecls : argdecl COMMA argdecls",
"argdecls : argdecl opt_dotdotdot",
"argdecl : type NAME",
"argdecl : type",
"args : OP opt_argdefines CP",
"opt_argdefines : argdefines",
"opt_argdefines :",
"argdefines : argdefine COMMA argdefines",
"argdefines : argdefine opt_dotdotdot",
"argdefine : opt_type NAME",
"argdefine : type",
"opt_dotdotdot : DOTDOTDOT",
"opt_dotdotdot :",
"opt_expr : expr",
"opt_expr :",
"expr : comma_expr",
"expr : decl initnames",
"comma_expr : simpleexpr",
"comma_expr : comma_expr COMMA simpleexpr",
"opt_exprs : exprs",
"opt_exprs :",
"exprs : simpleexpr COMMA exprs",
"exprs : simpleexpr",
"opt_actuals : actuals",
"opt_actuals :",
"actuals : simpleexpr COMMA actuals",
"actuals : simpleexpr opt_dotdotdot",
"simpleexpr : primary",
"simpleexpr : simpleexpr assignop simpleexpr",
"simpleexpr : MOD integer",
"simpleexpr : TIMES simpleexpr",
"simpleexpr : LAND simpleexpr",
"simpleexpr : MINUS simpleexpr",
"simpleexpr : LNOT simpleexpr",
"simpleexpr : BANG simpleexpr",
"simpleexpr : simpleexpr BANG",
"simpleexpr : INC simpleexpr",
"simpleexpr : simpleexpr INC",
"simpleexpr : DEC simpleexpr",
"simpleexpr : simpleexpr DEC",
"simpleexpr : FORK simpleexpr",
"simpleexpr : simpleexpr PLUS simpleexpr",
"simpleexpr : simpleexpr MINUS simpleexpr",
"simpleexpr : simpleexpr TIMES simpleexpr",
"simpleexpr : simpleexpr DIVIDE simpleexpr",
"simpleexpr : simpleexpr DIV simpleexpr",
"simpleexpr : simpleexpr MOD simpleexpr",
"simpleexpr : simpleexpr POW simpleexpr",
"simpleexpr : simpleexpr SHIFTL simpleexpr",
"simpleexpr : simpleexpr SHIFTR simpleexpr",
"simpleexpr : simpleexpr QUEST simpleexpr COLON simpleexpr",
"simpleexpr : simpleexpr LXOR simpleexpr",
"simpleexpr : simpleexpr LAND simpleexpr",
"simpleexpr : simpleexpr LOR simpleexpr",
"simpleexpr : simpleexpr AND simpleexpr",
"simpleexpr : simpleexpr OR simpleexpr",
"simpleexpr : simpleexpr EQ simpleexpr",
"simpleexpr : simpleexpr NE simpleexpr",
"simpleexpr : simpleexpr LT simpleexpr",
"simpleexpr : simpleexpr GT simpleexpr",
"simpleexpr : simpleexpr LE simpleexpr",
"simpleexpr : simpleexpr GE simpleexpr",
"assignop : ASSIGNPLUS",
"assignop : ASSIGNMINUS",
"assignop : ASSIGNTIMES",
"assignop : ASSIGNDIVIDE",
"assignop : ASSIGNDIV",
"assignop : ASSIGNMOD",
"assignop : ASSIGNPOW",
"assignop : ASSIGNSHIFTL",
"assignop : ASSIGNSHIFTR",
"assignop : ASSIGNLXOR",
"assignop : ASSIGNLAND",
"assignop : ASSIGNLOR",
"assignop : ASSIGN",
"primary : fullname",
"primary : TEN_NUM",
"primary : OCTAL_NUM",
"primary : OCTAL0_NUM",
"primary : BINARY_NUM",
"primary : HEX_NUM",
"primary : TEN_FLOAT",
"primary : OCTAL_FLOAT",
"primary : OCTAL0_FLOAT",
"primary : BINARY_FLOAT",
"primary : HEX_FLOAT",
"primary : CHAR_CONST",
"primary : STRING_CONST",
"primary : VOIDVAL",
"primary : BOOLVAL",
"primary : OS CS",
"primary : OP type CP namespace_start init namespace_end",
"primary : OP OS stars CS CP namespace_start arrayinit namespace_end",
"primary : OP OS dims CS CP namespace_start opt_arrayinit namespace_end",
"primary : OP OS type CS CP namespace_start opt_hashinit namespace_end",
"primary : type DOT NAME",
"primary : OP type DOT NAME CP primary",
"primary : DOLLAR opt_integer",
"primary : DOT",
"primary : OP expr CP",
"primary : OP block CP",
"primary : primary STAROS dims CS",
"primary : primary OS dims CS",
"primary : primary OP opt_actuals CP",
"primary : primary DOT NAME",
"primary : primary ARROW NAME",
"primary : opt_type FUNC namespace_start args func_body namespace_end",
"opt_integer : integer",
"opt_integer :",
"integer : TEN_NUM",
"integer : OCTAL_NUM",
"integer : OCTAL0_NUM",
"integer : BINARY_NUM",
"integer : HEX_NUM",
"opt_arrayinit : arrayinit",
"opt_arrayinit : OC CC",
"opt_arrayinit :",
"arrayinit : OC arrayelts opt_comma opt_dotdotdot CC",
"arrayinit : OC OS namespace_start compnames comparray CS comprehension namespace_end CC",
"comprehension : ASSIGN arrayelt",
"comprehension : block",
"compnames : compname COMMA compnames",
"compnames : compname",
"compname : NAME",
"comparray :",
"arrayelts : arrayelts COMMA arrayelt",
"arrayelts : arrayelt",
"arrayelt : simpleexpr",
"arrayelt : init",
"opt_hashinit : hashinit",
"opt_hashinit : OC CC",
"opt_hashinit :",
"hashinit : OC hashelts opt_comma CC",
"hashelts : hashelts COMMA hashelt",
"hashelts : hashelt",
"hashelt : simpleexpr DARROW hashvalue",
"hashelt : DARROW hashvalue",
"hashvalue : simpleexpr",
"hashvalue : init",
"structinit : OC structelts opt_comma CC",
"structelts : structelts COMMA structelt",
"structelts : structelt",
"structelt : NAME ASSIGN simpleexpr",
"structelt : NAME ASSIGN init",
"init : arrayinit",
"init : structinit",
"init : hashinit",
"init : OC CC",
};
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 500
#define YYMAXDEPTH 500
#endif
#endif
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short yyss[YYSTACKSIZE];
YYSTYPE yyvs[YYSTACKSIZE];
#define yystacksize YYSTACKSIZE
#line 1506 "gram.y"

static void
DeclListMark (void *object)
{
    DeclListPtr	dl = object;

    MemReference (dl->next);
    MemReference (dl->init);
}

DataType DeclListType = { DeclListMark, 0, "DeclListType" };

DeclListPtr
NewDeclList (Atom name, ExprPtr init, DeclListPtr next)
{
    ENTER ();
    DeclListPtr	dl;

    dl = ALLOCATE (&DeclListType, sizeof (DeclList));
    dl->next = next;
    dl->name = name;
    dl->symbol = 0;
    dl->init = init;
    RETURN (dl);
}

static void
MemListMark (void *object)
{
    MemListPtr	ml = object;

    MemReference (ml->next);
    MemReference (ml->type);
    MemReference (ml->atoms);
}

DataType MemListType = { MemListMark, 0, "MemListType" };

MemListPtr
NewMemList (AtomListPtr atoms, Type *type, MemListPtr next)
{
    ENTER ();
    MemListPtr	ml;

    ml = ALLOCATE (&MemListType, sizeof (MemList));
    ml->next = next;
    ml->type = type;
    ml->atoms = atoms;
    RETURN (ml);
}


extern NamespacePtr	CurrentNamespace;
FramePtr	CurrentFrame;

Value
lookupVar (char *ns, char *n)
{
    ENTER ();
    Value	    v;
    SymbolPtr	    symbol;
    NamespacePtr    namespace;
    Bool	    search;

    search = True;
    namespace = CurrentNamespace;
    if (ns)
    {
	symbol = NamespaceFindName (CurrentNamespace, AtomId (ns), True);
	if (symbol && symbol->symbol.class == class_namespace)
	    namespace = symbol->namespace.namespace;
	else
	    namespace = 0;
    }
    if (namespace)
	symbol = NamespaceFindName (namespace, AtomId (n), search);
    else
	symbol = 0;
    if (symbol && symbol->symbol.class == class_global)
	v = BoxValue (symbol->global.value, 0);
    else
	v = Void;
    RETURN (v);
}

void
setVar (NamespacePtr namespace, char *n, Value v, Type *type)
{
    ENTER ();
    Atom	atom;
    SymbolPtr	symbol;

    atom = AtomId (n);
    symbol = NamespaceFindName (namespace, atom, True);
    if (!symbol)
	symbol = NamespaceAddName (namespace,
				   NewSymbolGlobal (atom, type),
				   publish_private);
    if (symbol->symbol.class == class_global)
	BoxValueSet (symbol->global.value, 0, v);
    EXIT ();
}

void
GetNamespace (NamespacePtr *scope, FramePtr *fp)
{
    *scope = CurrentNamespace;
    *fp = CurrentFrame;
}

ExprPtr
BuildName (char *ns, char *n)
{
    ENTER ();
    SymbolPtr	    symbol, ns_symbol = 0;
    Atom	    atom, ns_atom = 0;
    Bool	    search = True;
    NamespacePtr    namespace = CurrentNamespace;
    ExprPtr	    e;
    Bool	    ns_privateFound = False;
    Bool	    privateFound = False;

    if (ns)
    {
	ns_atom = AtomId (ns);
	ns_symbol = NamespaceFindName (namespace, ns_atom, search);
	if (ns_symbol && ns_symbol->symbol.class == class_namespace)
	    namespace = ns_symbol->namespace.namespace;
	else
	{
	    if (!ns_symbol)
		ns_privateFound = NamespaceIsNamePrivate (namespace, ns_atom, search);
	    namespace = 0;
	}
	search = False;
    }
    atom = AtomId (n);
    if (namespace)
    {
	symbol = NamespaceFindName (namespace, atom, search);
	if (!symbol)
	    privateFound = NamespaceIsNamePrivate (namespace, atom, search);
    }
    else
	symbol = 0;
    e = NewExprAtom (atom, symbol, privateFound);
    if (ns_atom)
	e = NewExprTree (COLONCOLON, NewExprAtom (ns_atom, ns_symbol, ns_privateFound), e);
    RETURN (e);
}

static Value
AtomString (Atom id)
{
    ENTER ();
    RETURN (NewStrString (AtomName (id)));
}

ExprPtr
BuildRawname (ExprPtr colonnames, Atom name)
{
    ENTER ();
    int     len;
    Value   array;
    ExprPtr e;

    len = 1;
    for (e = colonnames; e; e = e->tree.left)
	len++;
    array = NewArray (False, False, typePrim[rep_string], 1, &len);
    len--;
    ArrayValueSet (&array->array, len, AtomString (name));
    e = colonnames;
    while (--len >= 0)
    {
	ArrayValueSet (&array->array, len, AtomString (e->tree.right->atom.atom));
	e = e->tree.left;
    }
    e = NewExprConst (POLY_CONST, array);
    RETURN (e);
}

	    
ExprPtr
BuildFullname (ExprPtr left, Atom atom)
{
    ENTER ();
    NamespacePtr    namespace;
    SymbolPtr	    symbol;
    Bool	    search;
    ExprPtr	    nameExpr;
    Bool	    privateFound = False;

    if (left)
    {
	if (left->base.tag == COLONCOLON)
	    symbol = left->tree.right->atom.symbol;
	else
	    symbol = left->atom.symbol;
	if (symbol && symbol->symbol.class == class_namespace)
	    namespace = symbol->namespace.namespace;
	else
	    namespace = 0;
        search = False;
    }
    else
    {
	namespace = CurrentNamespace;
	search = True;
    }
    if (namespace)
    {
	symbol = NamespaceFindName (namespace, atom, search);
	if (!symbol)
	    privateFound = NamespaceIsNamePrivate (namespace, atom, search);
    }
    else
	symbol = 0;
    nameExpr = NewExprAtom (atom, symbol, privateFound);
    if (left)
	nameExpr = NewExprTree (COLONCOLON, left, nameExpr);
    RETURN (nameExpr);
}

ExprPtr
BuildCall (char *scope, char *name, int nargs, ...)
{
    ENTER ();
    va_list	    alist;
    ExprPtr	    args, *prev;
    ExprPtr	    f;
    ExprPtr	    e;

    va_start (alist, nargs);
    prev = &args;
    args = 0;
    while (nargs--)
    {
	*prev = NewExprTree (COMMA, va_arg (alist, ExprPtr), 0);
	prev = &(*prev)->tree.right;
    }
    va_end (alist);
    f = BuildName (scope, name);
    e = NewExprTree (OP, f, args);
#ifdef DEBUG
    printExpr (stdout, e, -1, 0);
    printf ("\n");
#endif
    RETURN (e);
}

/*
 * Walk for() loop arguments and normalize the list
 */
static Expr *
ParseCanonFor (Expr *expr)
{
    if (!expr || !expr->tree.right) {
        ParseError ("Too few exprs in for()\n");
        return 0;
    }
    if (!expr->tree.right->tree.right) {
        /* 2-argument for() */
        expr = NewExprTree(FOR, 0, expr);
    }
    if (expr->tree.right->tree.right->tree.right) {
        ParseError ("Too many exprs in for()\n");
        return 0;
    }
    return expr;
}

/*
 * Walk a type structure and resolve any type names
 */
static CanonTypeResult
ParseCanonType (TypePtr type, Bool forwardAllowed)
{
    ArgType	    *arg;
    int		    n;
    CanonTypeResult ret = CanonTypeDefined, t;
    Bool	    anyResolved;
    
    if (!type)
    {
	ParseError ("Type missing inside compiler");
	return False;
    }
    switch (type->base.tag) {
    case type_prim:
	break;
    case type_name:
	if (!TypeNameType(type))
	{
	    if (!type->name.name)
	    {
		ExprPtr e;
		e = type->name.expr;
		if (e->base.tag == COLONCOLON)
		    e = e->tree.right;
		type->name.name = e->atom.symbol;
		if (!type->name.name)
		{
		    ParseError ("No typedef \"%A\" in namespace",
				e->atom.atom);
		    ret = CanonTypeUndefined;
		    break;
		}
	    }
	    if (type->name.name->symbol.class != class_typedef)
	    {
		ParseError ("Symbol \"%A\" not a typedef", 
			    type->name.name->symbol.name);
		ret = CanonTypeUndefined;
	    }
	    else if (!TypeNameType(type))
	    {
		if (!forwardAllowed)
		    ParseError ("Typedef \"%A\" not defined yet", 
				type->name.name->symbol.name);
		ret = CanonTypeForward;
	    }
	    else
	    {
		ret = ParseCanonType (TypeNameType(type), forwardAllowed);
	    }
	}
	break;
    case type_ref:
	ret = ParseCanonType (type->ref.ref, True);
	if (ret == CanonTypeForward)
	    ret = CanonTypeDefined;
	break;
    case type_func:
	if (type->func.ret)
	    ret = ParseCanonType (type->func.ret, forwardAllowed);
	for (arg = type->func.args; arg; arg = arg->next)
	{
	    t = ParseCanonType (arg->type, forwardAllowed);
	    if (t < ret)
		ret = t;
	}
	break;
    case type_array:
	ret = ParseCanonType (type->array.type, forwardAllowed);
	break;
    case type_hash:
	ret = ParseCanonType (type->hash.type, forwardAllowed);
	t = ParseCanonType (type->hash.keyType, forwardAllowed);
	if (t < ret)
	    ret = t;
	break;
    case type_struct:
	for (n = 0; n < type->structs.structs->nelements; n++)
	{
	    StructType   *st = type->structs.structs;

	    t = ParseCanonType (BoxTypesElements(st->types)[n], forwardAllowed);
	    if (t < ret)
		ret = t;
	}
	break;
    case type_union:
	anyResolved = False;
	for (n = 0; n < type->structs.structs->nelements; n++)
	{
	    StructType   *st = type->structs.structs;

	    t = ParseCanonType (BoxTypesElements(st->types)[n], True);
	    if (t < ret)
		ret = t;
	    if (t == CanonTypeDefined)
		anyResolved = True;
	}
	if (ret == CanonTypeForward)
	{
	    if (anyResolved)
		ret = CanonTypeDefined;
	    else if (!forwardAllowed)
		ParseError ("No member of '%T' defined yet", type);
	}
	break;
    case type_types:
	break;
    }
    return ret;
}

static SymbolPtr
ParseNewSymbol (Publish publish, Class class, Type *type, Atom name)
{
    ENTER ();
    SymbolPtr	s = 0;
    
    if (class == class_undef)
	class = funcDepth ? class_auto : class_global;

    if (class == class_namespace || 
	(class == class_typedef && type == 0) || 	
	ParseCanonType (type, False) == CanonTypeDefined)
    {
	switch (class) {
	case class_const:
	    s = NewSymbolConst (name, type);
	    break;
	case class_global:
	    s = NewSymbolGlobal (name, type);
	    break;
	case class_static:
	    s = NewSymbolStatic (name, type);
	    break;
	case class_arg:
	    s = NewSymbolArg (name, type);
	    break;
	case class_auto:
	    s = NewSymbolAuto (name, type);
	    break;
	case class_exception:
	    s = NewSymbolException (name, type);
	    break;
	case class_typedef:
	    /*
	     * Special case for typedefs --
	     * allow forward declaration of untyped
	     * typedef names, then hook the
	     * new type to the old name
	     */
	    if (type)
	    {
		s = NamespaceFindName (CurrentNamespace, name, False);
		if (s && s->symbol.class == class_typedef && !s->symbol.type)
		{
		    s->symbol.type = type;
		    RETURN (s);
		}
	    }
	    s = NewSymbolType (name, type);
	    break;
	case class_namespace:
	    s = NewSymbolNamespace (name, NewNamespace (CurrentNamespace));
	    break;
	case class_undef:
	    break;
	}
	if (s)
	    NamespaceAddName (CurrentNamespace, s, publish);
    }
    RETURN (s);
}

int
yywrap (void)
{
    if (LexInteractive())
	printf ("\n");
    if (CurrentFrame)
    {
	do_Debug_done ();
	return 0;
    }
    return 1;
}

extern	char *yytext;

void
ParseError (char *fmt, ...)
{
    va_list	args;

    if (LexFileName ())
	FilePrintf (FileStderr, "%A:%d: ",
		    LexFileName (), LexFileLine ());
    va_start (args, fmt);
    FileVPrintf (FileStderr, fmt, args);
    FilePrintf (FileStderr, "\n");
    va_end (args);
}

void
yyerror (char *msg)
{
    ParseError ("%s before %S", msg, yytext);
}
#line 2476 "gram.c"
#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse()
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register char *yys;
    extern char *getenv();

    if (yys = getenv("YYDEBUG"))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if (yyn = yydefred[yystate]) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yyss + yystacksize - 1)
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#ifdef lint
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#ifdef lint
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yyss + yystacksize - 1)
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 5:
#line 201 "gram.y"
{ yyerrok; }
break;
case 6:
#line 203 "gram.y"
{ yyerrok; }
break;
case 7:
#line 211 "gram.y"
{ ignorenl++; }
break;
case 8:
#line 214 "gram.y"
{ ignorenl--; }
break;
case 9:
#line 217 "gram.y"
{ 
			ignorenl = 0; 
			LexNamespace = 0; 
			CurrentNamespace = TopNamespace; 
			funcDepth = 0; 
			notCommand = 0;
		    }
break;
case 10:
#line 227 "gram.y"
{ notCommand = 1; }
break;
case 17:
#line 242 "gram.y"
{
			ENTER ();
			ExprPtr	e;
			Value	t;
			NamespacePtr	s;
			FramePtr	f;
			
			e = BuildCall ("Command", "display",
				       1,NewExprTree (EXPR, yyvsp[-2].expr, 0));
			GetNamespace (&s, &f);
			t = NewThread (f, CompileExpr (e, 0));
			ThreadsRun (t, 0);
			EXIT ();
		    }
break;
case 18:
#line 257 "gram.y"
{
			ENTER ();
			ExprPtr	e;
			Value	t;
			NamespacePtr	s;
			FramePtr	f;

			e = BuildCall("Command", "display_base",
				      2, NewExprTree (EXPR, yyvsp[-4].expr, 0), yyvsp[-2].expr);
			GetNamespace (&s, &f);
			t = NewThread (f, CompileExpr (e, 0));
			ThreadsRun (t, 0);
			EXIT ();
		    }
break;
case 19:
#line 272 "gram.y"
{ 
			ENTER ();
			NamespacePtr    s;
			FramePtr    f;
			Value	    t;
			
			GetNamespace (&s, &f);
			t = NewThread (f, CompileStat (yyvsp[-2].expr, 0));
			ThreadsRun (t, 0);
			EXIT ();
		    }
break;
case 20:
#line 284 "gram.y"
{
			ENTER();
			ExprPtr	e;
			Value	t;
			NamespacePtr	s;
			FramePtr	f;
			CommandPtr	c;

			c = CommandFind (CurrentCommands, yyvsp[-4].atom);
			if (!c)
			    ParseError ("Undefined command \"%s\"", AtomName (yyvsp[-4].atom));
			else
			{
			    e = NewExprTree (OP, 
					     NewExprConst (POLY_CONST, c->func),
					     yyvsp[-3].expr);
			    GetNamespace (&s, &f);
			    t = NewThread (f, CompileExpr (e, 0));
			    ThreadsRun (t, 0);
			}
			EXIT ();
		    }
break;
case 21:
#line 307 "gram.y"
{
			ENTER ();
			ExprPtr e;

			Value	t;
			NamespacePtr	s;
			FramePtr	f;
			CommandPtr	c;

			c = CommandFind (CurrentCommands, yyvsp[-4].atom);
			if (!c)
			    ParseError ("Undefined command \"%s\"", AtomName (yyvsp[-4].atom));
			else
			{
			    e = NewExprTree (OP, 
					     NewExprConst (POLY_CONST, c->func),
					     yyvsp[-3].expr);
			    GetNamespace (&s, &f);
			    t = NewThread (f, CompileExpr (e, 0));
			    ThreadsRun (t, 0);
			}
			EXIT ();
		    }
break;
case 24:
#line 334 "gram.y"
{ yyval.expr = 0; }
break;
case 25:
#line 337 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 26:
#line 339 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[0].expr, 0); }
break;
case 27:
#line 342 "gram.y"
{ yyval.expr = BuildRawname (yyvsp[-1].expr, yyvsp[0].atom); }
break;
case 31:
#line 349 "gram.y"
{ yyval.expr = NewExprTree (COLONCOLON, yyvsp[-2].expr, NewExprAtom (yyvsp[-1].atom, 0, False)); }
break;
case 32:
#line 351 "gram.y"
{ yyval.expr = 0; }
break;
case 33:
#line 354 "gram.y"
{ 
			yyval.expr = BuildFullname (yyvsp[-1].expr, yyvsp[0].atom);
			LexNamespace = 0;
		    }
break;
case 34:
#line 359 "gram.y"
{ 
			yyval.expr = BuildFullname (0, yyvsp[0].atom);
			LexNamespace = 0;
		    }
break;
case 35:
#line 365 "gram.y"
{ 
			yyval.expr = BuildFullname (yyvsp[-1].expr, yyvsp[0].atom);
			LexNamespace = 0;
		    }
break;
case 36:
#line 370 "gram.y"
{
			yyval.expr = BuildFullname (0, yyvsp[0].atom);
			LexNamespace = 0;
		    }
break;
case 37:
#line 376 "gram.y"
{ 
			ExprPtr	    e;
			SymbolPtr   symbol;

			e = BuildFullname (yyvsp[-2].expr, yyvsp[-1].atom);
			if (e->base.tag == COLONCOLON)
			    symbol = e->tree.right->atom.symbol;
			else
			    symbol = e->atom.symbol;
			if (!symbol)
			{
			    ParseError ("non-existant namespace \"%A\"", yyvsp[-1].atom);
			    YYERROR;
			}
			else if (symbol->symbol.class != class_namespace)
			{
			    ParseError ("%A is not a namespace", yyvsp[-1].atom);
			    YYERROR;
			}
			LexNamespace = symbol->namespace.namespace;
			yyval.expr = e;
		    }
break;
case 38:
#line 399 "gram.y"
{ 
			ExprPtr	    e;
			SymbolPtr   symbol;

			e = BuildFullname (0, yyvsp[-1].atom);
			if (e->base.tag == COLONCOLON)
			    symbol = e->tree.right->atom.symbol;
			else
			    symbol = e->atom.symbol;
			if (!symbol)
			{
			    ParseError ("non-existant namespace \"%A\"", yyvsp[-1].atom);
			    YYERROR;
			}
			else if (symbol->symbol.class != class_namespace)
			{
			    ParseError ("%A is not a namespace", yyvsp[-1].atom);
			    YYERROR;
			}
			LexNamespace = symbol->namespace.namespace;
			yyval.expr = e;
		    }
break;
case 41:
#line 429 "gram.y"
{ CurrentNamespace = NewNamespace (CurrentNamespace); }
break;
case 42:
#line 432 "gram.y"
{ CurrentNamespace = CurrentNamespace->previous; }
break;
case 43:
#line 435 "gram.y"
{ yyval.expr = yyvsp[-1].expr; }
break;
case 46:
#line 442 "gram.y"
{ yyval.expr = NewExprTree(OC, yyvsp[-1].expr, yyvsp[0].expr); }
break;
case 47:
#line 444 "gram.y"
{ yyval.expr = NewExprTree(OC, 0, 0); }
break;
case 48:
#line 447 "gram.y"
{ yyval.expr = NewExprTree(IF, yyvsp[-4].expr, yyvsp[-2].expr); }
break;
case 49:
#line 449 "gram.y"
{ yyval.expr = NewExprTree(ELSE, yyvsp[-6].expr, NewExprTree(ELSE, yyvsp[-4].expr, yyvsp[-2].expr)); }
break;
case 50:
#line 451 "gram.y"
{ yyval.expr = NewExprTree(WHILE, yyvsp[-4].expr, yyvsp[-2].expr); }
break;
case 51:
#line 453 "gram.y"
{ yyval.expr = NewExprTree(DO, yyvsp[-6].expr, yyvsp[-3].expr); }
break;
case 52:
#line 455 "gram.y"
{ Expr *expr = ParseCanonFor(yyvsp[-4].expr);
		      if (!expr)
                          YYERROR;
		      yyval.expr = NewExprTree(FOR, expr, yyvsp[-2].expr); }
break;
case 53:
#line 460 "gram.y"
{ yyval.expr = NewExprTree (SWITCH, yyvsp[-4].expr, yyvsp[-2].expr); }
break;
case 54:
#line 462 "gram.y"
{ yyval.expr = NewExprTree (UNION, yyvsp[-4].expr, yyvsp[-2].expr); }
break;
case 55:
#line 464 "gram.y"
{ yyval.expr = NewExprTree(BREAK, (Expr *) 0, (Expr *) 0); }
break;
case 56:
#line 466 "gram.y"
{ yyval.expr = NewExprTree(CONTINUE, (Expr *) 0, (Expr *) 0); }
break;
case 57:
#line 468 "gram.y"
{ yyval.expr = NewExprTree (RETURNTOK, (Expr *) 0, yyvsp[-1].expr); }
break;
case 58:
#line 470 "gram.y"
{ yyval.expr = NewExprTree(EXPR, yyvsp[-1].expr, (Expr *) 0); }
break;
case 60:
#line 473 "gram.y"
{ yyval.expr = NewExprTree(SEMI, (Expr *) 0, (Expr *) 0); }
break;
case 61:
#line 475 "gram.y"
{ 
			DeclList    *decl = yyvsp[-2].funcDecl.decl;
			SymbolPtr   symbol = decl->symbol;
			Class	    class = yyvsp[-2].funcDecl.type.class;
			Publish	    publish = yyvsp[-2].funcDecl.type.publish;
			TypePtr	    type = yyvsp[-2].funcDecl.type.type;
			TypePtr	    ret = type->func.ret;
			ArgType	    *argType = type->func.args;

			if (symbol)
			{
			    if (yyvsp[-1].expr)
			    {
				symbol->symbol.forward = False;
				ParseCanonType (ret, False);
				decl->init = NewExprCode (NewFuncCode (ret,
								       argType,
								       yyvsp[-1].expr),
							  NewExprAtom (symbol->symbol.name, symbol, False));
			    }
			    else
				symbol->symbol.forward = True;
			}
			yyval.expr = NewExprDecl (FUNC, decl, class, type, publish);
		    }
break;
case 62:
#line 501 "gram.y"
{ 
			DeclListPtr decl;

			decl = NewDeclList (yyvsp[-5].atom, 0, 0);
			decl->symbol = ParseNewSymbol (yyvsp[-8].publish, 
						       class_exception, 
						       typePoly, yyvsp[-5].atom);
			yyval.expr = NewExprDecl (EXCEPTION,
					  decl,
					  class_exception,
					  NewTypeFunc (typePoly, yyvsp[-3].argType),
					  yyvsp[-8].publish);
		    }
break;
case 63:
#line 515 "gram.y"
{ yyval.expr = NewExprTree (RAISE, yyvsp[-4].expr, yyvsp[-2].expr); }
break;
case 64:
#line 517 "gram.y"
{ 
			DeclListPtr decl;

			for (decl = yyvsp[-2].declList; decl; decl = decl->next)
			    decl->symbol = ParseNewSymbol (yyvsp[-5].publish, class_typedef,
							   0, decl->name);
			yyval.expr = NewExprTree (TYPEDEF, NewExprDecl (TYPEDEF, yyvsp[-2].declList, class_typedef, 0, yyvsp[-5].publish), 0);
		    }
break;
case 65:
#line 526 "gram.y"
{ 
			DeclListPtr decl;

			for (decl = yyvsp[-2].declList; decl; decl = decl->next)
			    decl->symbol = ParseNewSymbol (yyvsp[-6].publish, class_typedef,
							   yyvsp[-3].type, decl->name);
			yyval.expr = NewExprTree (TYPEDEF, NewExprDecl (TYPEDEF, yyvsp[-2].declList, class_typedef, yyvsp[-3].type, yyvsp[-6].publish), 0);
		    }
break;
case 66:
#line 535 "gram.y"
{
			SymbolPtr   symbol;
			Publish	    publish = yyvsp[-3].publish;
			
			/*
			 * this is a hack - save the current namespace
			 * on the parser stack to be restored after
			 * the block is compiled.
			 */
			yyvsp[-2].namespace = CurrentNamespace;
			if (publish == publish_extend)
			{
			    symbol = NamespaceFindName (CurrentNamespace, yyvsp[0].atom, True);
			    if (!symbol)
			    {
				ParseError ("non-existant namespace %A", yyvsp[0].atom);
				YYERROR;
			    }
			    else if (symbol->symbol.class != class_namespace)
			    {
				ParseError ("%A is not a namespace", yyvsp[0].atom);
				YYERROR;
			    }
			    else
				CurrentNamespace = symbol->namespace.namespace;
			}
			else
			{
			    symbol = ParseNewSymbol (yyvsp[-3].publish, class_namespace,
						     0, yyvsp[0].atom);
			    CurrentNamespace = NewNamespace (CurrentNamespace);
			    symbol->namespace.namespace = CurrentNamespace;
			}
			/*
			 * Make all of the symbols visible while compiling within
			 * the namespace
			 */
			if (CurrentNamespace != yyvsp[-2].namespace)
			    CurrentNamespace->publish = publish_public;
		    }
break;
case 67:
#line 576 "gram.y"
{
			/*
			 * close the namespace to non-public lookups
			 */
			if (CurrentNamespace != yyvsp[-7].namespace)
			    CurrentNamespace->publish = publish_private;
			/*
			 * Restore to the namespace saved on
			 * the parser stack
			 */
			CurrentNamespace = yyvsp[-7].namespace;
			yyval.expr = NewExprTree (NAMESPACE, NewExprAtom (yyvsp[-5].atom, 0, False), yyvsp[-2].expr);
		    }
break;
case 68:
#line 590 "gram.y"
{
			SymbolPtr	symbol;
			ExprPtr		e;

			e = yyvsp[-2].expr;
			if (yyvsp[-2].expr->base.tag == COLONCOLON)
			    e = e->tree.right;
			symbol = e->atom.symbol;
			if (!symbol)
			{
			    ParseError ("non-existant namespace %A", e->atom.atom);
			    YYERROR;
			}
			else if (symbol->symbol.class != class_namespace)
			{
			    ParseError ("%A is not a namespace", e->atom.atom);
			    YYERROR;
			}
			NamespaceImport (CurrentNamespace, 
					 symbol->namespace.namespace, yyvsp[-5].publish);
			yyval.expr = NewExprTree (IMPORT, yyvsp[-2].expr, NewExprDecl (IMPORT,
								   0, 
								   class_undef,
								   0,
								   yyvsp[-5].publish));
		    }
break;
case 69:
#line 617 "gram.y"
{ yyval.expr = NewExprTree (CATCH, yyvsp[-1].expr, yyvsp[-2].expr); }
break;
case 70:
#line 619 "gram.y"
{ yyval.expr = NewExprTree (TWIXT, 
					NewExprTree (TWIXT, yyvsp[-6].expr, yyvsp[-4].expr),
					NewExprTree (TWIXT, yyvsp[-2].expr, 0));
		    }
break;
case 71:
#line 625 "gram.y"
{ yyval.expr = NewExprTree(SEMI, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 72:
#line 627 "gram.y"
{ yyval.expr = NewExprTree(SEMI, yyvsp[0].expr, 0); }
break;
case 73:
#line 630 "gram.y"
{ yyval.expr = NewExprTree (CATCH, yyvsp[-1].expr, yyvsp[0].expr); }
break;
case 74:
#line 632 "gram.y"
{ yyval.expr = 0; }
break;
case 75:
#line 635 "gram.y"
{ yyval.expr = NewExprCode (NewFuncCode (typePrim[rep_void], yyvsp[-2].argType, yyvsp[-1].expr), yyvsp[-4].expr); }
break;
case 77:
#line 639 "gram.y"
{ yyval.expr = 0; }
break;
case 78:
#line 641 "gram.y"
{ ++funcDepth; }
break;
case 79:
#line 641 "gram.y"
{ --funcDepth; yyval.expr = yyvsp[0].expr; }
break;
case 80:
#line 644 "gram.y"
{ yyval.expr = yyvsp[0].expr; }
break;
case 81:
#line 646 "gram.y"
{ 
			yyval.expr = NewExprTree (OC,
					  NewExprTree (RETURNTOK, 0, yyvsp[-1].expr),
					  NewExprTree (OC, 0, 0));
		    }
break;
case 82:
#line 653 "gram.y"
{ yyval.expr = yyvsp[-1].expr; }
break;
case 83:
#line 656 "gram.y"
{ yyval.expr = NewExprTree (CASE, yyvsp[-1].expr, yyvsp[0].expr); }
break;
case 84:
#line 658 "gram.y"
{ yyval.expr = 0; }
break;
case 85:
#line 661 "gram.y"
{ yyval.expr = NewExprTree (CASE, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 86:
#line 663 "gram.y"
{ yyval.expr = NewExprTree (CASE, 0, yyvsp[0].expr); }
break;
case 87:
#line 666 "gram.y"
{ yyval.expr = yyvsp[-1].expr; }
break;
case 88:
#line 669 "gram.y"
{ yyval.expr = NewExprTree (CASE, yyvsp[-1].expr, yyvsp[0].expr); }
break;
case 89:
#line 671 "gram.y"
{ yyval.expr = 0; }
break;
case 90:
#line 674 "gram.y"
{ 
			SymbolPtr   sym = yyvsp[-3].symbol;
			Atom	    sym_atom = sym ? sym->symbol.name : 0;
			ExprPtr	    name = 0;
			
			if (sym)
			    name = NewExprAtom (sym_atom, sym, False);
			    
			yyval.expr = NewExprTree (CASE, 
					  NewExprTree (CASE,
						       NewExprAtom (yyvsp[-4].atom, 0, False),
						       name),
					  yyvsp[-1].expr);
		    }
break;
case 91:
#line 689 "gram.y"
{ yyval.expr = NewExprTree (CASE, 0, yyvsp[0].expr); }
break;
case 92:
#line 692 "gram.y"
{
			yyval.symbol = ParseNewSymbol (publish_private,
					     class_undef,
					     typePoly,
					     yyvsp[0].atom);
		    }
break;
case 93:
#line 699 "gram.y"
{ yyval.symbol = 0; }
break;
case 94:
#line 705 "gram.y"
{ yyval.atomList = NewAtomList (yyvsp[0].atomList, yyvsp[-2].atom); }
break;
case 95:
#line 707 "gram.y"
{ yyval.atomList = NewAtomList (0, yyvsp[0].atom); }
break;
case 96:
#line 710 "gram.y"
{ yyval.declList = NewDeclList (yyvsp[-2].atom, 0, yyvsp[0].declList); }
break;
case 97:
#line 712 "gram.y"
{ yyval.declList = NewDeclList (yyvsp[0].atom, 0, 0); }
break;
case 100:
#line 725 "gram.y"
{ 
			if (yyvsp[-3].symbol)
			{
			    yyval.declList = NewDeclList (yyvsp[-3].symbol->symbol.name, yyvsp[-2].expr, yyvsp[0].declList); 
			    yyval.declList->symbol = yyvsp[-3].symbol;
			}
			else
			    yyval.declList = yyvsp[0].declList;
		    }
break;
case 101:
#line 735 "gram.y"
{ 
			if (yyvsp[-1].symbol)
			{
			    yyval.declList = NewDeclList (yyvsp[-1].symbol->symbol.name, yyvsp[0].expr, 0);
			    yyval.declList->symbol = yyvsp[-1].symbol;
			}
			else
			    yyvsp[-1].symbol = 0;
		    }
break;
case 102:
#line 746 "gram.y"
{	
			yyval.symbol = ParseNewSymbol (yyvsp[-1].fulltype.publish,
					     yyvsp[-1].fulltype.class,
					     yyvsp[-1].fulltype.type,
					     yyvsp[0].atom);
		    }
break;
case 103:
#line 758 "gram.y"
{ yyval.fulltype = yyvsp[-3].fulltype; }
break;
case 104:
#line 764 "gram.y"
{
			NamespacePtr	save = CurrentNamespace;
			SymbolPtr	symbol;
			Type		*type = NewTypeFunc (yyvsp[-3].funcDecl.type.type, yyvsp[-1].argType);
			/*
			 * namespace_start pushed a new namespace, make sure
			 * this symbol is placed in the original namespace
			 */
			CurrentNamespace = save->previous;
			symbol = NamespaceFindName (CurrentNamespace, yyvsp[-3].funcDecl.decl->name, True);
			if (symbol && symbol->symbol.forward)
			{
			    if (!TypeIsSupertype (symbol->symbol.type, type))
			    {
				ParseError ("%A redefinition with different type",
					    yyvsp[-3].funcDecl.decl->name);
				symbol = 0;
			    }
			}
			else
			{
			    symbol = ParseNewSymbol (yyvsp[-3].funcDecl.type.publish,
						     yyvsp[-3].funcDecl.type.class, 
						     type,
						     yyvsp[-3].funcDecl.decl->name);
			}
			CurrentNamespace = save;
			yyval.funcDecl = yyvsp[-3].funcDecl;
			yyval.funcDecl.decl->symbol = symbol;
			yyval.funcDecl.type.type = type;
		    }
break;
case 105:
#line 797 "gram.y"
{ yyval.funcDecl.type = yyvsp[-2].fulltype; yyval.funcDecl.decl = NewDeclList (yyvsp[-1].atom, 0, 0); }
break;
case 106:
#line 799 "gram.y"
{ yyval.funcDecl.type = yyvsp[-3].fulltype; yyval.funcDecl.decl = NewDeclList (yyvsp[-1].atom, 0, 0); }
break;
case 107:
#line 801 "gram.y"
{ 
			yyval.funcDecl.type.publish = publish_private;
			yyval.funcDecl.type.class = class_undef;
			yyval.funcDecl.type.type = typePoly;
			yyval.funcDecl.decl = NewDeclList (yyvsp[-1].atom, 0, 0); 
		    }
break;
case 108:
#line 809 "gram.y"
{ yyval.expr = yyvsp[0].expr; }
break;
case 109:
#line 811 "gram.y"
{ yyval.expr = yyvsp[0].expr; }
break;
case 110:
#line 813 "gram.y"
{ yyval.expr = 0; }
break;
case 111:
#line 819 "gram.y"
{ yyval.fulltype.publish = yyvsp[-4].publish; yyval.fulltype.class = yyvsp[-3].class; yyval.fulltype.type = yyvsp[-2].type; }
break;
case 112:
#line 821 "gram.y"
{ yyval.fulltype.publish = publish_private; yyval.fulltype.class = yyvsp[-3].class; yyval.fulltype.type = yyvsp[-2].type; }
break;
case 113:
#line 823 "gram.y"
{ yyval.fulltype.publish = yyvsp[-3].publish; yyval.fulltype.class = class_undef; yyval.fulltype.type = yyvsp[-2].type; }
break;
case 114:
#line 825 "gram.y"
{ yyval.fulltype.publish = publish_private; yyval.fulltype.class = class_undef; yyval.fulltype.type = yyvsp[-2].type; }
break;
case 115:
#line 827 "gram.y"
{ yyval.fulltype.publish = yyvsp[-3].publish; yyval.fulltype.class = yyvsp[-2].class; yyval.fulltype.type = typePoly; }
break;
case 116:
#line 829 "gram.y"
{ yyval.fulltype.publish = publish_private; yyval.fulltype.class = yyvsp[-2].class; yyval.fulltype.type = typePoly; }
break;
case 117:
#line 831 "gram.y"
{ yyval.fulltype.publish = yyvsp[-2].publish; yyval.fulltype.class = class_undef; yyval.fulltype.type = typePoly; }
break;
case 118:
#line 837 "gram.y"
{ yyval.type = NewTypeFunc (yyvsp[0].type, yyvsp[-1].argType); }
break;
case 119:
#line 839 "gram.y"
{ yyval.type = NewTypeArray (yyvsp[0].type, yyvsp[-2].expr, False); }
break;
case 120:
#line 841 "gram.y"
{ yyval.type = NewTypeArray (yyvsp[0].type, yyvsp[-2].expr, True); }
break;
case 121:
#line 843 "gram.y"
{ yyval.type = NewTypeArray (yyvsp[0].type, yyvsp[-2].expr, False); }
break;
case 122:
#line 845 "gram.y"
{ yyval.type = NewTypeHash (yyvsp[0].type, yyvsp[-2].type); }
break;
case 123:
#line 847 "gram.y"
{ yyval.type = 0; }
break;
case 124:
#line 850 "gram.y"
{
			Type	*top = yyvsp[0].type;
			Type	*t, **bot = &top;

			/*
			 * Walk down the type chain to hang the
			 * base type off of the end.  This
			 * makes int[]() be an array of functions
			 * rather than a function returning an array
			 */
			while ((t = *bot))
			    switch (t->base.tag) {
			    case type_array:
				bot = &t->array.type;
				break;
			    case type_hash:
				bot = &t->hash.type;
				break;
			    case type_func:
				bot = &t->func.ret;
				break;
			    default:
				assert(0);
			    }
			*bot = yyvsp[-1].type;
			yyval.type = top;
		    }
break;
case 125:
#line 878 "gram.y"
{ yyval.type = NewTypeRef (yyvsp[0].type, True); }
break;
case 126:
#line 880 "gram.y"
{ yyval.type = NewTypeRef (yyvsp[0].type, False); }
break;
case 128:
#line 884 "gram.y"
{
			AtomListPtr	al;
			StructType	*st;
			MemListPtr	ml;
			int		nelements;

			nelements = 0;
			for (ml = yyvsp[-1].memList; ml; ml = ml->next)
			{
			    for (al = ml->atoms; al; al = al->next)
				nelements++;
			}
			st = NewStructType (nelements);
			nelements = 0;
			for (ml = yyvsp[-1].memList; ml; ml = ml->next)
			{
			    for (al = ml->atoms; al; al = al->next)
			    {
				AddBoxType (&st->types, ml->type);
				StructTypeAtoms(st)[nelements] = al->atom;
				nelements++;
			    }
			}
			yyval.type = NewTypeStruct (st);
		    }
break;
case 129:
#line 910 "gram.y"
{
			AtomListPtr	al;
			StructType	*st;
			MemListPtr	ml;
			int		nelements;

			nelements = 0;
			for (ml = yyvsp[-1].memList; ml; ml = ml->next)
			{
			    for (al = ml->atoms; al; al = al->next)
				nelements++;
			}
			st = NewStructType (nelements);
			nelements = 0;
			for (ml = yyvsp[-1].memList; ml; ml = ml->next)
			{
			    for (al = ml->atoms; al; al = al->next)
			    {
				AddBoxType (&st->types, ml->type);
				StructTypeAtoms(st)[nelements] = al->atom;
				nelements++;
			    }
			}
			yyval.type = NewTypeUnion (st, False);
		    }
break;
case 130:
#line 936 "gram.y"
{
			AtomListPtr	al;
			StructType	*st;
			int		nelements;

			nelements = 0;
			for (al = yyvsp[-1].atomList; al; al = al->next)
			    nelements++;
			
			st = NewStructType (nelements);
			nelements = 0;
			for (al = yyvsp[-1].atomList; al; al = al->next)
			{
			    AddBoxType (&st->types, typePrim[rep_void]);
			    StructTypeAtoms(st)[nelements] = al->atom;
			    nelements++;
			}
			yyval.type = NewTypeUnion (st, True);
		    }
break;
case 131:
#line 957 "gram.y"
{ yyval.type = yyvsp[-1].type; }
break;
case 132:
#line 959 "gram.y"
{ yyval.type = NewTypeName (yyvsp[0].expr, 0); }
break;
case 134:
#line 963 "gram.y"
{ yyval.type = typePoly; }
break;
case 148:
#line 980 "gram.y"
{ yyval.expr = 0; }
break;
case 149:
#line 983 "gram.y"
{ yyval.expr = NewExprTree (COMMA, 0, yyvsp[-2].expr); }
break;
case 150:
#line 985 "gram.y"
{ yyval.expr = NewExprTree (COMMA, 0, 0); }
break;
case 151:
#line 988 "gram.y"
{ yyval.expr = NewExprTree (COMMA, 0, yyvsp[-2].expr); }
break;
case 152:
#line 990 "gram.y"
{ yyval.expr = NewExprTree (COMMA, 0, 0); }
break;
case 153:
#line 993 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 154:
#line 995 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[0].expr, 0); }
break;
case 155:
#line 1001 "gram.y"
{ yyval.memList = NewMemList (yyvsp[-2].atomList, yyvsp[-3].type, yyvsp[0].memList); }
break;
case 156:
#line 1003 "gram.y"
{ yyval.memList = 0; }
break;
case 157:
#line 1006 "gram.y"
{ yyval.memList = NewMemList (yyvsp[-2].atomList, yyvsp[-3].type, yyvsp[0].memList); }
break;
case 158:
#line 1008 "gram.y"
{ yyval.memList = 0; }
break;
case 164:
#line 1021 "gram.y"
{ yyval.publish = publish_private; }
break;
case 169:
#line 1035 "gram.y"
{ yyval.argType = yyvsp[-1].argType; }
break;
case 170:
#line 1037 "gram.y"
{ yyval.argType = 0; }
break;
case 171:
#line 1040 "gram.y"
{ yyval.argType = NewArgType (yyvsp[-2].argDecl.type, False, yyvsp[-2].argDecl.name, 0, yyvsp[0].argType); }
break;
case 172:
#line 1042 "gram.y"
{ yyval.argType = NewArgType (yyvsp[-1].argDecl.type, yyvsp[0].bool, yyvsp[-1].argDecl.name, 0, 0); }
break;
case 173:
#line 1045 "gram.y"
{ 
			ParseCanonType (yyvsp[-1].type, False);
			yyval.argDecl.type = yyvsp[-1].type; 
			yyval.argDecl.name = yyvsp[0].atom; 
		    }
break;
case 174:
#line 1051 "gram.y"
{ 
			ParseCanonType (yyvsp[0].type, False);
			yyval.argDecl.type = yyvsp[0].type;
			yyval.argDecl.name = 0; 
		    }
break;
case 175:
#line 1062 "gram.y"
{ yyval.argType = yyvsp[-1].argType; }
break;
case 176:
#line 1065 "gram.y"
{
			ArgType	*args;
			Type	*type;

			for (args = yyvsp[0].argType; args; args = args->next)
			{
			    type = args->type;
			    if (ParseCanonType (type, False) != CanonTypeDefined)
				break;
			    if (args->varargs)
			    {
				type = NewTypeArray (type,
						     NewExprTree (COMMA,
								  NewExprConst (TEN_NUM, 
										NewInt (0)),
								  0),
						     False);
			    }
			    args->symbol = ParseNewSymbol (publish_private,
							   class_arg, 
							   type, args->name);
			}
			yyval.argType = yyvsp[0].argType;
		    }
break;
case 177:
#line 1090 "gram.y"
{ yyval.argType = 0; }
break;
case 178:
#line 1093 "gram.y"
{ yyval.argType = NewArgType (yyvsp[-2].argDecl.type, False, yyvsp[-2].argDecl.name, 0, yyvsp[0].argType); }
break;
case 179:
#line 1095 "gram.y"
{ yyval.argType = NewArgType (yyvsp[-1].argDecl.type, yyvsp[0].bool, yyvsp[-1].argDecl.name, 0, 0); }
break;
case 180:
#line 1098 "gram.y"
{ yyval.argDecl.type = yyvsp[-1].type; yyval.argDecl.name = yyvsp[0].atom; }
break;
case 181:
#line 1100 "gram.y"
{ yyval.argDecl.type = yyvsp[0].type; yyval.argDecl.name = 0; }
break;
case 182:
#line 1103 "gram.y"
{ yyval.bool = True; }
break;
case 183:
#line 1105 "gram.y"
{ yyval.bool = False; }
break;
case 185:
#line 1113 "gram.y"
{ yyval.expr = 0; }
break;
case 187:
#line 1117 "gram.y"
{ 
			DeclList    *decl;

			for (decl = yyvsp[0].declList; decl; decl = decl->next)
			{
			    if (decl->init)
			    {
				if (!decl->init->base.type)
				    decl->init->base.type = yyvsp[-1].fulltype.type;
			    }
			}

			yyval.expr = NewExprDecl (VAR, yyvsp[0].declList, yyvsp[-1].fulltype.class, yyvsp[-1].fulltype.type, yyvsp[-1].fulltype.publish); 
		    }
break;
case 189:
#line 1134 "gram.y"
{ yyval.expr = NewExprTree(COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 191:
#line 1141 "gram.y"
{ yyval.expr = 0; }
break;
case 192:
#line 1144 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 193:
#line 1146 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[0].expr, 0); }
break;
case 195:
#line 1150 "gram.y"
{ yyval.expr = 0; }
break;
case 196:
#line 1153 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 197:
#line 1155 "gram.y"
{	
			ExprPtr	arg = yyvsp[0].bool ? NewExprTree (DOTDOTDOT, yyvsp[-1].expr, 0) : yyvsp[-1].expr;
			yyval.expr = NewExprTree (COMMA, arg, 0); 
		    }
break;
case 199:
#line 1165 "gram.y"
{ 
			if (yyvsp[-1].ints == ASSIGNPOW)
			    yyval.expr = NewExprTree (ASSIGNPOW, 
					      BuildName ("Math", "assign_pow"),
					      NewExprTree (ASSIGNPOW, yyvsp[-2].expr, yyvsp[0].expr));
			else
			{
			    ExprPtr left = yyvsp[-2].expr;
			    /*
			     * Automatically declare names used in
			     * simple assignements at the top level
			     */
			    if (yyvsp[-1].ints == ASSIGN && 
				funcDepth == 0 &&
				left->base.tag == NAME && 
				!(left->atom.symbol))
			    {
				yyvsp[-2].expr->atom.symbol = ParseNewSymbol (publish_private,
								  class_undef, 
								  typePoly, 
								  yyvsp[-2].expr->atom.atom);
			    }
			    yyval.expr = NewExprTree(yyvsp[-1].ints, yyvsp[-2].expr, yyvsp[0].expr); 
			}
		    }
break;
case 200:
#line 1191 "gram.y"
{   Value	t;
			t = do_Thread_id_to_thread (yyvsp[0].value);
			if (t == Void)
			{
			    ParseError ("No thread %v", yyvsp[0].value);
			    YYERROR;
			}
			else
			    yyval.expr = NewExprConst(THREAD_CONST, t); 
		    }
break;
case 201:
#line 1202 "gram.y"
{ yyval.expr = NewExprTree(STAR, yyvsp[0].expr, (Expr *) 0); }
break;
case 202:
#line 1204 "gram.y"
{ yyval.expr = NewExprTree(AMPER, yyvsp[0].expr, (Expr *) 0); }
break;
case 203:
#line 1206 "gram.y"
{ yyval.expr = NewExprTree(UMINUS, yyvsp[0].expr, (Expr *) 0); }
break;
case 204:
#line 1208 "gram.y"
{ yyval.expr = NewExprTree(LNOT, yyvsp[0].expr, (Expr *) 0); }
break;
case 205:
#line 1210 "gram.y"
{ yyval.expr = NewExprTree(BANG, yyvsp[0].expr, (Expr *) 0); }
break;
case 206:
#line 1212 "gram.y"
{ yyval.expr = NewExprTree(FACT, yyvsp[-1].expr, (Expr *) 0); }
break;
case 207:
#line 1214 "gram.y"
{ yyval.expr = NewExprTree(INC, yyvsp[0].expr, (Expr *) 0); }
break;
case 208:
#line 1216 "gram.y"
{ yyval.expr = NewExprTree(INC, (Expr *) 0, yyvsp[-1].expr); }
break;
case 209:
#line 1218 "gram.y"
{ yyval.expr = NewExprTree(DEC, yyvsp[0].expr, (Expr *) 0); }
break;
case 210:
#line 1220 "gram.y"
{ yyval.expr = NewExprTree(DEC, (Expr *) 0, yyvsp[-1].expr); }
break;
case 211:
#line 1222 "gram.y"
{ yyval.expr = NewExprTree (FORK, (Expr *) 0, yyvsp[0].expr); }
break;
case 212:
#line 1224 "gram.y"
{ yyval.expr = NewExprTree(PLUS, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 213:
#line 1226 "gram.y"
{ yyval.expr = NewExprTree(MINUS, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 214:
#line 1228 "gram.y"
{ yyval.expr = NewExprTree(TIMES, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 215:
#line 1230 "gram.y"
{ yyval.expr = NewExprTree(DIVIDE, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 216:
#line 1232 "gram.y"
{ yyval.expr = NewExprTree(DIV, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 217:
#line 1234 "gram.y"
{ yyval.expr = NewExprTree(MOD, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 218:
#line 1236 "gram.y"
{ 
			yyval.expr = NewExprTree(POW, 
					 BuildName ("Math", "pow"), 
					 NewExprTree (POW, yyvsp[-2].expr, yyvsp[0].expr)); 
		    }
break;
case 219:
#line 1242 "gram.y"
{ yyval.expr = NewExprTree(SHIFTL, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 220:
#line 1244 "gram.y"
{ yyval.expr = NewExprTree(SHIFTR, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 221:
#line 1246 "gram.y"
{ yyval.expr = NewExprTree(QUEST, yyvsp[-4].expr, NewExprTree(COLON, yyvsp[-2].expr, yyvsp[0].expr)); }
break;
case 222:
#line 1248 "gram.y"
{ yyval.expr = NewExprTree(LXOR, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 223:
#line 1250 "gram.y"
{ yyval.expr = NewExprTree(LAND, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 224:
#line 1252 "gram.y"
{ yyval.expr = NewExprTree(LOR, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 225:
#line 1254 "gram.y"
{ yyval.expr = NewExprTree(AND, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 226:
#line 1256 "gram.y"
{ yyval.expr = NewExprTree(OR, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 227:
#line 1258 "gram.y"
{ yyval.expr = NewExprTree(EQ, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 228:
#line 1260 "gram.y"
{ yyval.expr = NewExprTree(NE, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 229:
#line 1262 "gram.y"
{ yyval.expr = NewExprTree(LT, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 230:
#line 1264 "gram.y"
{ yyval.expr = NewExprTree(GT, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 231:
#line 1266 "gram.y"
{ yyval.expr = NewExprTree(LE, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 232:
#line 1268 "gram.y"
{ yyval.expr = NewExprTree(GE, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 247:
#line 1286 "gram.y"
{ yyval.expr = NewExprConst(TEN_NUM, yyvsp[0].value); }
break;
case 248:
#line 1288 "gram.y"
{ yyval.expr = NewExprConst(OCTAL_NUM, yyvsp[0].value); }
break;
case 249:
#line 1290 "gram.y"
{ yyval.expr = NewExprConst(OCTAL0_NUM, yyvsp[0].value); }
break;
case 250:
#line 1292 "gram.y"
{ yyval.expr = NewExprConst(BINARY_NUM, yyvsp[0].value); }
break;
case 251:
#line 1294 "gram.y"
{ yyval.expr = NewExprConst(HEX_NUM, yyvsp[0].value); }
break;
case 252:
#line 1296 "gram.y"
{ yyval.expr = NewExprConst(TEN_FLOAT, yyvsp[0].value); }
break;
case 253:
#line 1298 "gram.y"
{ yyval.expr = NewExprConst(OCTAL_FLOAT, yyvsp[0].value); }
break;
case 254:
#line 1300 "gram.y"
{ yyval.expr = NewExprConst(OCTAL0_FLOAT, yyvsp[0].value); }
break;
case 255:
#line 1302 "gram.y"
{ yyval.expr = NewExprConst(BINARY_FLOAT, yyvsp[0].value); }
break;
case 256:
#line 1304 "gram.y"
{ yyval.expr = NewExprConst(HEX_FLOAT, yyvsp[0].value); }
break;
case 257:
#line 1306 "gram.y"
{ yyval.expr = NewExprConst(CHAR_CONST, yyvsp[0].value); }
break;
case 258:
#line 1308 "gram.y"
{ yyval.expr = NewExprConst(STRING_CONST, yyvsp[0].value); }
break;
case 259:
#line 1310 "gram.y"
{ yyval.expr = NewExprConst(VOIDVAL, yyvsp[0].value); }
break;
case 260:
#line 1312 "gram.y"
{ yyval.expr = NewExprConst(BOOLVAL, yyvsp[0].value); }
break;
case 261:
#line 1314 "gram.y"
{ 
			yyval.expr = BuildFullname (0, AtomId ("[]"));
		    }
break;
case 262:
#line 1318 "gram.y"
{ 
			ParseCanonType (yyvsp[-4].type, False);
			if (yyvsp[-1].expr)
			    yyvsp[-1].expr->base.type = yyvsp[-4].type;
			yyval.expr = NewExprTree (NEW, yyvsp[-1].expr, 0);
			yyval.expr->base.type = yyvsp[-4].type;
		    }
break;
case 263:
#line 1326 "gram.y"
{ 
			yyvsp[-1].expr->base.type = NewTypeArray (typePoly, yyvsp[-5].expr, True); 
			ParseCanonType (yyvsp[-1].expr->base.type, False);
			yyval.expr = NewExprTree (NEW, yyvsp[-1].expr, 0); 
			yyval.expr->base.type = yyvsp[-1].expr->base.type;
		    }
break;
case 264:
#line 1333 "gram.y"
{ 
			yyvsp[-1].expr->base.type = NewTypeArray (typePoly, yyvsp[-5].expr, False);
			ParseCanonType (yyvsp[-1].expr->base.type, False);
			yyval.expr = NewExprTree (NEW, yyvsp[-1].expr, 0); 
			yyval.expr->base.type = yyvsp[-1].expr->base.type;
		    }
break;
case 265:
#line 1340 "gram.y"
{
			yyvsp[-1].expr->base.type = NewTypeHash (typePoly, yyvsp[-5].type);
			ParseCanonType (yyvsp[-1].expr->base.type, False);
			yyval.expr = NewExprTree (NEW, yyvsp[-1].expr, 0);
			yyval.expr->base.type = yyvsp[-1].expr->base.type;
		    }
break;
case 266:
#line 1347 "gram.y"
{
			ParseCanonType (yyvsp[-2].type, False);
			yyval.expr = NewExprTree (UNION, NewExprAtom (yyvsp[0].atom, 0, False), 0); 
			yyval.expr->base.type = yyvsp[-2].type;
		    }
break;
case 267:
#line 1353 "gram.y"
{ 
			ParseCanonType (yyvsp[-4].type, False);
			yyval.expr = NewExprTree (UNION, NewExprAtom (yyvsp[-2].atom, 0, False), yyvsp[0].expr); 
			yyval.expr->base.type = yyvsp[-4].type;
		    }
break;
case 268:
#line 1359 "gram.y"
{ yyval.expr = BuildCall ("History", "fetch", 1, NewExprConst (TEN_NUM, yyvsp[0].value)); }
break;
case 269:
#line 1361 "gram.y"
{ yyval.expr = NewExprTree (DOLLAR, 0, 0); }
break;
case 270:
#line 1363 "gram.y"
{ yyval.expr = yyvsp[-1].expr; }
break;
case 271:
#line 1365 "gram.y"
{ yyval.expr = yyvsp[-1].expr; }
break;
case 272:
#line 1367 "gram.y"
{ yyval.expr = NewExprTree (OS, NewExprTree (STAR, yyvsp[-3].expr, (Expr *) 0), yyvsp[-1].expr); }
break;
case 273:
#line 1369 "gram.y"
{ yyval.expr = NewExprTree(OS, yyvsp[-3].expr, yyvsp[-1].expr); }
break;
case 274:
#line 1371 "gram.y"
{ yyval.expr = NewExprTree (OP, yyvsp[-3].expr, yyvsp[-1].expr); }
break;
case 275:
#line 1373 "gram.y"
{ yyval.expr = NewExprTree(DOT, yyvsp[-2].expr, NewExprAtom (yyvsp[0].atom, 0, False)); }
break;
case 276:
#line 1375 "gram.y"
{ yyval.expr = NewExprTree(ARROW, yyvsp[-2].expr, NewExprAtom (yyvsp[0].atom, 0, False)); }
break;
case 277:
#line 1377 "gram.y"
{ 
			ParseCanonType (yyvsp[-5].type, False);
			yyval.expr = NewExprCode (NewFuncCode (yyvsp[-5].type, yyvsp[-2].argType, yyvsp[-1].expr), 0); 
		    }
break;
case 279:
#line 1384 "gram.y"
{ yyval.value = Zero; }
break;
case 286:
#line 1397 "gram.y"
{ yyval.expr = 0; }
break;
case 287:
#line 1399 "gram.y"
{ yyval.expr = 0; }
break;
case 288:
#line 1402 "gram.y"
{ 
			ExprPtr	elts = yyvsp[-3].expr ? ExprRehang (yyvsp[-3].expr, 0) : 0;
			if (yyvsp[-1].bool)
			{
			    ExprPtr i = elts;
			    while (i->tree.right)
				i = i->tree.right;
			    i->tree.right = NewExprTree (COMMA, 
							 NewExprTree (DOTDOTDOT, 0, 0),
							 0);
			}
			yyval.expr = NewExprTree (ARRAY, elts, 0); 
		    }
break;
case 289:
#line 1416 "gram.y"
{
			yyval.expr = NewExprTree (COMP, NewExprTree (COMP, yyvsp[-5].expr, yyvsp[-4].expr), yyvsp[-2].expr);
		    }
break;
case 290:
#line 1421 "gram.y"
{ yyval.expr = yyvsp[0].expr; }
break;
case 292:
#line 1425 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 294:
#line 1429 "gram.y"
{ 
			SymbolPtr   s;
			s = ParseNewSymbol (publish_private, class_arg, 
					    typePrim[rep_integer], yyvsp[0].atom);
			yyval.expr = NewExprAtom (yyvsp[0].atom, s, False); 
		    }
break;
case 295:
#line 1437 "gram.y"
{
			SymbolPtr   s;
			Atom	    a = AtomId("[]");
			s = ParseNewSymbol (publish_private, class_undef,
					    typePoly, a);
			yyval.expr = NewExprAtom (a, s, False);
		    }
break;
case 296:
#line 1446 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 297:
#line 1448 "gram.y"
{ yyval.expr = NewExprTree (COMMA, 0, yyvsp[0].expr); }
break;
case 301:
#line 1459 "gram.y"
{ yyval.expr = 0; }
break;
case 302:
#line 1461 "gram.y"
{ yyval.expr = 0; }
break;
case 303:
#line 1464 "gram.y"
{
			ExprPtr elts = yyvsp[-2].expr ? ExprRehang (yyvsp[-2].expr, 0) : 0;
			yyval.expr = NewExprTree (HASH, elts, 0);
		    }
break;
case 304:
#line 1470 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 305:
#line 1472 "gram.y"
{ yyval.expr = NewExprTree (COMMA, 0, yyvsp[0].expr); }
break;
case 306:
#line 1475 "gram.y"
{ yyval.expr = NewExprTree (DARROW, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 307:
#line 1477 "gram.y"
{ yyval.expr = NewExprTree (DARROW, 0, yyvsp[0].expr); }
break;
case 310:
#line 1486 "gram.y"
{ yyval.expr = NewExprTree (STRUCT, ExprRehang (yyvsp[-2].expr, 0), 0); }
break;
case 311:
#line 1489 "gram.y"
{ yyval.expr = NewExprTree (COMMA, yyvsp[-2].expr, yyvsp[0].expr); }
break;
case 312:
#line 1491 "gram.y"
{ yyval.expr = NewExprTree (COMMA, 0, yyvsp[0].expr); }
break;
case 313:
#line 1494 "gram.y"
{ yyval.expr = NewExprTree (ASSIGN, NewExprAtom (yyvsp[-2].atom, 0, False), yyvsp[0].expr); }
break;
case 314:
#line 1496 "gram.y"
{ yyval.expr = NewExprTree (ASSIGN, NewExprAtom (yyvsp[-2].atom, 0, False), yyvsp[0].expr); }
break;
case 318:
#line 1503 "gram.y"
{ yyval.expr = NewExprTree (ANONINIT, 0, 0); }
break;
#line 4140 "gram.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yyss + yystacksize - 1)
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
